/**
 * @file SecretKey.cpp
 * @brief Implementation of reader the secret key which contains the methods required for parse the encrypted file.
 * @date 05/04/2011
 * @author Marcel Cordovi
 */

#include "SecretKey.h"
#include "Configuration/s-a-e-s-common-conf-manager.h"
#include "System/Security/Cryptography/PBEKeyDerivatorHMAC.h"
#include <Security/KeyManager.hxx>
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <System/IO/File.h>
#include <System/Encoding/Base64Encoder.h>
#include <System/Security/Cryptography/MsgDigestUtil.h>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define HMAC_KEY_LEN 32 ///< hmac key length

using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * SecretKey::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Common.Security.SecretKey", GETTEXT_PACKAGE );

// ================= Utility ==========================

void SecretKey::mClearKey ( Smartmatic::System::SafeBuffer & key )
{
    key.Clear();
}

void SecretKey::mAssignKey ( const Smartmatic::System::SafeBuffer & keyIn,
                             Smartmatic::System::SafeBuffer & keyOut )
{
    keyOut.Clear();
    keyOut.AddData ( keyIn );
}

// ================= Operator Overloading ======================

SecretKey & SecretKey::operator= ( const SecretKey & sk )
{
    if ( this != &sk )
    {
        // clear current state
        mClearKey ( mp_DecryptionKey );
        mClearKey ( mp_SecretKey );
        mClearKey ( mp_PassphraseEncrypted );
        // copy new assigned values
        this->m_FilePath.assign ( sk.m_FilePath );
        this->m_IsFileEncrypted = sk.m_IsFileEncrypted;
        this->m_Passphrase = sk.m_Passphrase;
        this->m_rnd = sk.m_rnd;
        this->m_votingCenterCode = sk.m_votingCenterCode;
        mAssignKey (
            sk.mp_DecryptionKey
            , this->mp_DecryptionKey );
        mAssignKey (
            sk.mp_SecretKey
            , this->mp_SecretKey );
        mAssignKey (
            sk.mp_PassphraseEncrypted
            , this->mp_PassphraseEncrypted );
    }

    return *this;
}


// ================= Access =======================================

std::string SecretKey::getKeyBase64 () const
{
    return mp_SecretKey.GetBase64Data();
}

void SecretKey::getKey ( unsigned char *& key, int & keyLen )
{
    key = ( unsigned char * ) mp_SecretKey.GetData();
    keyLen = mp_SecretKey.GetSize();
}

const unsigned char * SecretKey::getKey ( int & keyLen ) const
{
    keyLen = mp_SecretKey.GetSize();
    return mp_SecretKey.GetData();
}

const unsigned char * SecretKey::getPassphrase ( size_t & bufferLen ) const
{
    bufferLen = mp_PassphraseEncrypted.GetSize();
    return mp_PassphraseEncrypted.GetData();
}


// ================= Life Cycle ==================================

SecretKey::SecretKey()
{
    m_IsFileEncrypted = true;
}

SecretKey::SecretKey ( const SecretKey & sk )
{
    this->m_FilePath.assign ( sk.m_FilePath );
    this->m_IsFileEncrypted = sk.m_IsFileEncrypted;
    this->m_Passphrase = sk.m_Passphrase;
    this->m_rnd = sk.m_rnd;
    this->m_votingCenterCode = sk.m_votingCenterCode;

    mAssignKey (
        sk.mp_DecryptionKey
        , this->mp_DecryptionKey );
    mAssignKey (
        sk.mp_SecretKey
        , this->mp_SecretKey );
    mAssignKey (
        sk.mp_PassphraseEncrypted
        , this->mp_PassphraseEncrypted );
}

SecretKey::SecretKey ( const std::string & filePath, const std::string & passPhrase, const std::string & votingCenterCode, const std::string & rnd)
throw ( SecretKeyException )
{
    processKey ( filePath, passPhrase, votingCenterCode, rnd );
}

void SecretKey::processKey ( const std::string & filePath, const std::string & passPhrase, const std::string & votingCenterCode, const std::string & rnd )
throw ( SecretKeyException )
{
    std::string passphrase ( passPhrase );
    m_Passphrase = passPhrase;
    m_rnd = rnd;
    m_votingCenterCode = votingCenterCode;
    /* State initialization */
    logger->Debug ( "Open file: " + filePath );
    m_FilePath.assign ( filePath );
    m_IsFileEncrypted = SAESCommonConfigurationManager::getInstance()->getConfiguration()->
                        getMachineSecuritySchema().getSecretKeyEncrypted();

    /* Calculate file's decryption key  */

    if ( m_IsFileEncrypted )
    {
        SafeBuffer salt;
        int iterations = 0;
        SaltSourceType salt_conf = SAESCommonConfigurationManager::getInstance()->getConfiguration()->
                                   getMachineSecuritySchema().getSaltSource();

        if ( salt_conf.getCalcSalt() ) // check whether salt must be calculated from the rnd and voting center code
        {
            std::string saltSeed;
            saltSeed=votingCenterCode + "-" + rnd;
            SafeBuffer dbg_saltSeed ( saltSeed );
            MsgDigestUtil digest ( dbg_saltSeed );
            digest.getDigest ( salt, MsgDigestUtil::DGSTUTIL_SHA256 );
            iterations = SAESCommonConfigurationManager::getInstance()->getConfiguration()->
                getMachineSecuritySchema().getIterations();
        }
        else // obtain salt the old way: A SHA256 hash of the passphrase
        {
            SafeBuffer dbg_salt ( passphrase );
            MsgDigestUtil digest ( dbg_salt );
            digest.getDigest ( salt, MsgDigestUtil::getDefaultSecretKeyDigest() );
            /* Debug: Prints calculated salt  */
            logger->Debug ( "Salt = " + salt.GetBase64Data() );
            iterations = SAESCommonConfigurationManager::getInstance()->getConfiguration()->
                         getMachineSecuritySchema().getIterations();
        }

        mObtainSymmetricKey (
            passphrase
            , salt
            , SAESCommonConfigurationManager::getInstance()->getConfiguration()->
            getMachineSecuritySchema().getKeyDerivAlg()
            , SAESCommonConfigurationManager::getInstance()->getConfiguration()->
            getMachineSecuritySchema().getDigestAlg()
            , iterations );
        /* Debug: Prints PBE key */
        logger->Debug ( "PBE Key = " + mp_DecryptionKey.GetBase64Data() );
    }

    mEncryptPassphrase ( passphrase );
    /* Extract key from file */
    mUnwrap ();
    /* Debug: Prints unencrypted file content */
    logger->Debug ( "----------------------------------------------------------------" );
    logger->Debug ( "Key = " + getKeyBase64 () );
    logger->Debug ( "----------------------------------------------------------------" );
}

SecretKey::~SecretKey()
{
    mClearKey ( mp_DecryptionKey );
    mClearKey ( mp_SecretKey );
    mClearKey ( mp_PassphraseEncrypted );
}


// ================= Key Processing ======================

void SecretKey::mProcessInputPassphrase (
    std::string & passphrase
    , std::string & salt
    , std::string sep_pattern
    , int & iterations )
throw ( SecretKeyException )
{
    std::vector<std::string> tokens;
    size_t num_tokens;

    if ( sep_pattern == "|" ) // TODO: use regular expression for regex's reserved tokens detection and scapeping
    {
        sep_pattern = "\\" + sep_pattern;    // preppend scape character
    }

    Utils::StringUtils::splitToVector ( passphrase, sep_pattern, tokens );

    num_tokens = tokens.size();

    if ( num_tokens == 0 || num_tokens > 3 )
    {
        throw SecretKeyException ( _ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorInvalidPassphraseFormat" ), N_ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorInvalidPassphraseFormat" ), logger );
    }

    passphrase.assign ( tokens[0] );

    if ( num_tokens >= 2 )
    {
        salt = tokens[1];
    }
    else // no provided
    {
        salt.assign ( "" );
    }

    if ( num_tokens == 3 )
    {
        iterations = atoi ( tokens[2].c_str () );
    }
    else // no provided
    {
        iterations = 0;
    }
}

void SecretKey::mObtainSymmetricKey (
    const std::string & passPhrase
    , const SafeBuffer & salt
    , const Smartmatic::SAES::Common::Configuration::KeyDerivAlgType drv_alg
    , const Smartmatic::SAES::Common::Configuration::DigestAlgType   dgst_alg
    , const int iterations )
throw ( SecretKeyException )
{
    SafeBuffer outputBuffer;

    // TODO: HMAC is the only algorithm supported for the moment

    try
    {
        switch ( drv_alg )
        {
            case KEYDERIV_HMAC:
                {
                    PBEKeyDerivatorHMAC deriver;
                    deriver.SetSalt ( salt );
                    deriver.GenerateKey (
                        passPhrase
                        , outputBuffer
                        , MsgDigestUtil::getDefaultGenerateKeyDigest()
                        , HMAC_KEY_LEN
                        , iterations );
                }
                break;

            case KEYDERIV_DES:
            case KEYDERIV_3KEY3DES:
            case KEYDERIV_2KEY3DES:
            case KEYDERIV_RC4:
                throw SecretKeyException ( _ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorAlgorithmNotSupported" ), N_ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorAlgorithmNotSupported" ), logger );
                break;

            default:
                throw SecretKeyException ( _ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorAlgorithmNotSupported" ), N_ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorAlgorithmNotSupported" ), logger );
        }
    }
    catch ( ... ) // TODO: catch HMAC exception
    {
        throw SecretKeyException ( _ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorDerivingDecryptionKey" ), N_ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorDerivingDecryptionKey" ), logger );
    }

    mAssignKey ( outputBuffer, mp_DecryptionKey );
}

void SecretKey::mUnwrap () throw ( SecretKeyException )
{
    if ( !File::getInstance()->canOpenFile ( m_FilePath ) )
    {
        throw SecretKeyException ( _ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorOpenFileSecretKey" ),
                                   N_ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorOpenFileSecretKey" ), logger );
    }

    SafeBuffer dataIn;
    File::getInstance()->openFileToBuffer ( m_FilePath, dataIn );

    if ( m_IsFileEncrypted )
    {
        try
        {
            Crypto crypt;
            crypt.init ( Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getKeySize(),
                         Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getModeType(), true );
            Smartmatic::System::SafeBuffer dataOut;
            crypt.decryptBufferToBufferAndGetIV(
                dataIn
                , dataOut
                , mp_DecryptionKey.GetData());

            mAssignKey (
                dataOut
                , mp_SecretKey );
        }
        catch ( CryptoException & ce )
        {
            throw SecretKeyException (
                _ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ),
                N_ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ), logger );
        }
    }
    else
    {
        SafeBuffer decode;
        Base64Encoder::ConvertFromBase64 ( dataIn.GetBase64Data(), decode );
        mAssignKey ( decode, mp_SecretKey );
    }
}

void SecretKey::mEncryptPassphrase ( const std::string & passphrase )
throw ( SecretKeyException )
{
    Smartmatic::System::SafeBuffer passEncrypted;
    Smartmatic::System::SafeBuffer data ( ( unsigned char * ) passphrase.data(), ( int ) passphrase.size () );

    Smartmatic::System::SafeBuffer initKey;

    Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getKey (
        Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_INIT_KEY, initKey );

    try
    {
        Crypto crypt;
        crypt.init ( Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getKeySize(),
                     Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getModeType(), true );
        crypt.encryptBufferToBufferAndCreateIV (
            data
            , passEncrypted
            , initKey.GetData()
            );
    }
    catch ( CryptoException & ce )
    {
        throw SecretKeyException (
            _ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ),
            N_ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ), logger );
    }

    mAssignKey (
        passEncrypted
        , mp_PassphraseEncrypted );
}

void SecretKey::getSafeBuffer ( Smartmatic::System::SafeBuffer & buffer )
{
    buffer.Clear();
    buffer.AddData ( mp_SecretKey );
}
