/**
 * @file	close-election-functionality.cc
 * @brief	close election functionality class.
 * @author	Gustavo Garcia
 * @date	31/01/2017
 */


#include "close-election-functionality.h"
#include <Voting/OperationStatus.hxx>
#include <Voting/Status/ElectionStatusWrapper.hxx>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Gui/Widgets/AbstractFactoryWidget.hxx"
#include <Gui/Widgets/FactoryMethodWidget.hxx>
#include <Operation/Devices/Polling/BCRPolling.hxx>

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Gtk;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Functionality;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Operation::Devices;

CloseElectionFunctionality * CloseElectionFunctionality::current_instance = NULL;

Smartmatic::Log::ISMTTLog * CloseElectionFunctionality::logger = Smartmatic::Log::SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality", GETTEXT_PACKAGE );

CloseElectionFunctionality * CloseElectionFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new CloseElectionFunctionality();
    }

    return current_instance;
}

CloseElectionFunctionality::CloseElectionFunctionality()
{
    persistAllResultWidget = NULL;
    closeStatus = NULL;
    closeConfirmation = NULL;
    manager = NULL;
    validateSignatureWidget = NULL;
    validateVotesWidget = NULL;
    count = 0;
    countGeneratePV = 0;
    countGenerateAllResult = 0;
    generatePV = false;
    resultsPackageTransmissionWidget = NULL;
    externalPrintWidget = NULL;
}

CloseElectionFunctionality::~CloseElectionFunctionality()
{
    deleteElements();
}

void CloseElectionFunctionality::deleteElements()
{
    if ( persistAllResultWidget )
    {
        delete persistAllResultWidget;
        persistAllResultWidget = NULL;
    }

    if ( resultsPackageTransmissionWidget )
    {
        delete resultsPackageTransmissionWidget;
        resultsPackageTransmissionWidget = NULL;
    }

    if ( closeStatus )
    {
        delete closeStatus;
        closeStatus = NULL;
    }

    if ( closeConfirmation )
    {
        delete closeConfirmation;
        closeConfirmation = NULL;
    }

    if ( manager )
    {
        delete manager;
        manager = NULL;
    }

    if ( validateSignatureWidget )
    {
        delete validateSignatureWidget;
        validateSignatureWidget = NULL;
    }

    if ( validateVotesWidget )
    {
        delete validateVotesWidget;
        validateVotesWidget = NULL;
    }

    if ( externalPrintWidget )
    {
        delete externalPrintWidget;
        externalPrintWidget = NULL;
    }
}

void CloseElectionFunctionality::Initialize()
{
    BCRPolling::getInstance()->pause ();
    MachineOperationManager::getInstance()->setWaitingConfirmationToCloseElection(true);
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.VotationWasNotClose" ) );
    disableExitButton();

    if ( manager )
    {
        manager->Start();
    }
}

void CloseElectionFunctionality::on_close_clicked()
{
    try
    {
        MachineOperationManager::getInstance()->closeElection();
    }
    catch ( Smartmatic::System::Exception::CryptoException & ex )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.ErrorClosingElection" ) );
    }
    catch ( Smartmatic::System::Exception::SignedException & ex )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.ErrorClosingElection" ) );
    }
    catch ( Smartmatic::System::Exception::XmlException & ex )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.ErrorClosingElection" ) );
    }
    catch ( ... )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.ErrorClosingElection" ) );
    }

    MachineOperationManager::getInstance()->setWaitingConfirmationToCloseElection(false);
    ExitFunctionality();
}

void CloseElectionFunctionality::LoadWizardFrame()
{
    CloseElectionFunctionalityConfiguration::ClosingElectionStepsSequence  seq =
        SAESGUIConfigurationManager::getInstance()->GetCloseElectionFunctionalityConfiguration().getClosingElectionSteps();
    Smartmatic::SAES::Configuration::CloseElectionFunctionalityConfiguration::ClosingElectionStepsIterator iter;
    countGeneratePV = -1;
    countGenerateAllResult = -1;
    generatePV = false;

    for ( iter = seq.begin(); iter != seq.end(); iter++ )
    {
        switch ( *iter )
        {
            default:
                break;

            case CloseElectionSteps::CONFIGURATION_CLOSECONFIRMATION:
                closeConfirmation = new CloseElectionConfirmationWidget();
                closeConfirmation->on_stop_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::on_cancelButton_clicked ) );
                closeConfirmation->on_disable_close_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::disableExitButton ) );
                wizardVector.push_back ( closeConfirmation );
                break;

            case CloseElectionSteps::CONFIGURATION_CLOSEELECTION:
                closeStatus = AbstractFactoryWidget::createCloseElectionStatusWidget();
                closeStatus->getSignalEnd().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::ElectionWasClosed ) );
                closeStatus->on_stop_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::ErrorHappen ) );
                closeStatus->on_disable_close_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::disableExitButton ) );
                wizardVector.push_back ( closeStatus );
                break;

            case CloseElectionSteps::CONFIGURATION_GENERATEALLRESULT:
                persistAllResultWidget = AbstractFactoryWidget::createPersistAllResultWidget();
                persistAllResultWidget->on_stop_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::on_cancelButton_clicked ) );
                persistAllResultWidget->on_disable_close_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::disableExitButton ) );
                countGenerateAllResult = wizardVector.size();
                wizardVector.push_back ( persistAllResultWidget );
                break;

            case CloseElectionSteps::CONFIGURATION_VALIDATEEMLSIGNATURE:
                validateSignatureWidget = AbstractFactoryWidget::createValidateEmlSignatureWidget();
                validateSignatureWidget->on_stop_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::on_cancelButton_clicked ) );
                validateSignatureWidget->on_disable_close_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::disableExitButton ) );
                wizardVector.push_back ( validateSignatureWidget );
                break;

            case CloseElectionSteps::CONFIGURATION_VALIDATEVOTES:
                validateVotesWidget = AbstractFactoryWidget::createValidateVotesWidget();
                validateVotesWidget->on_stop_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::on_cancelButton_clicked ) );
                validateVotesWidget->on_disable_close_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::disableExitButton ) );
                wizardVector.push_back ( validateVotesWidget );
                break;

            case CloseElectionSteps::CONFIGURATION_RESULTSPACKAGETRANSMISSION:
                resultsPackageTransmissionWidget = FactoryMethodWidget::createResultsPackageTransmissionWidget();
                resultsPackageTransmissionWidget->on_stop_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::on_cancelButton_clicked ) );
                resultsPackageTransmissionWidget->on_disable_close_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::disableExitButton ) );
                wizardVector.push_back ( resultsPackageTransmissionWidget );
                break;

            case CloseElectionSteps::CONFIGURATION_EXTERNALPRINTPV:
                externalPrintWidget = new ExternalPrintWidget();
                externalPrintWidget->on_stop_signal_emit().connect(sigc::mem_fun(*this, &CloseElectionFunctionality::on_cancelButton_clicked));
                externalPrintWidget->on_disable_close_signal_emit().connect(sigc::mem_fun(*this, &CloseElectionFunctionality::disableExitButton));
                wizardVector.push_back(externalPrintWidget);

                generatePV = true;

                break;
        }
    }

    manager = new WizardWidgetManager ( wizardVector );

    manager->on_transition_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::TransitionSignal ) );
    manager->on_finish_signal_emit().connect ( sigc::mem_fun ( *this, &CloseElectionFunctionality::endWizardMamager ) );
    manager->show_all_children();
    manager->setAccessibleName ( "CloseElectionWizardManager" );

    count = 1;
}

void CloseElectionFunctionality::ElectionWasClosed()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.ElectionWasClosed" ) );
    RefreshInfo();
}

void CloseElectionFunctionality::HideOnFinish()
{
    MachineOperationManager::getInstance()->setWaitingConfirmationToCloseElection(false);
    ExitFunctionality();
}

void CloseElectionFunctionality::on_cancelButton_clicked()
{
    MachineOperationManager::getInstance()->setWaitingConfirmationToCloseElection(false);
    BCRPolling::getInstance()->resume();
    ExitFunctionality();
}

Gtk::Widget * CloseElectionFunctionality::GetFrame()
{
    LoadWizardFrame();
    return manager;
}

void CloseElectionFunctionality::TransitionSignal()
{
    bool enableHeldButton = ( countGeneratePV == -1 || countGeneratePV != count ) &&
                            ( countGenerateAllResult == -1 || countGenerateAllResult != count );
    bool enableButton = ( ++count ) == ( int ) wizardVector.size() ||
                        manager->getCurrentWidget()->isEnabledButtonExitWizard();
    setExitButtonSensitive ( enableButton );
    setHelpButtonSensitive ( enableButton && enableHeldButton );
}

void CloseElectionFunctionality::ErrorHappen()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.ErrorClosingElection" ) );
    enableExitButton();
    RefreshInfo();
}

std::string CloseElectionFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.CloseElection" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType CloseElectionFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMCLOSEELECTIONFUNCTIONALITY;
}

void CloseElectionFunctionality::Dispose()
{
    wizardVector.clear();

    if ( MachineOperationManager::getInstance()->getOperationStatus()->getElectionStatusWrapper()->getEletionState()
            != Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_RESULTSTRANSMITTED )
    {
        MachineOperationManager::getInstance()->deleteCloseFiles();
    }
}

Smartmatic::Log::ISMTTLog * CloseElectionFunctionality::getLogger()
{
    return logger;
}

Glib::ustring CloseElectionFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.LogStartFunctionality" );
}

Glib::ustring CloseElectionFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.LogCloseFunctionality" );
}

void CloseElectionFunctionality::endWizardMamager ()
{
    if ( !generatePV )
    {
        MachineOperationManager::getInstance()->setWaitingConfirmationToCloseElection(false);
        ExitFunctionality();
    }
}
