/*
 * AbleDiagnosticSoundWidget.cxx
 *
 *  Created on: Feb 20, 2018
 *      Author: eduardo.fernandez
 */


#include "AbleDiagnosticSoundWidget.hxx"
#include "Configuration/label-configuration.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Environment/Configurations/resource-path-provider.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Widgets/Diagnose/AbleSoundDiagnosticSampleAudio.hxx>
#include <Gui/Widgets/FactoryMethodWidget.hxx>
#include <Operation/Voting/AudioVoting/AudioVotingCheckUtility.hxx>
#include <System/Audio/Audio.hxx>
#include <System/Audio/AudioPlayer.hxx>
#include <System/Audio/MixerControl.hxx>
#include <System/Exception/AudioException.hxx>

#ifdef HAVE_CONFIG_H
#include "../../../config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::System::Audio;
using namespace Smartmatic::System::Exception;

ISMTTLog * AbleDiagnosticSoundWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.AbleDiagnosticSoundWidget", GETTEXT_PACKAGE );

static const size_t BITS_PER_SAMPLE = 16;
static const size_t SAMPLES_PER_SECOND = 44100;
static const size_t CHANNELS = 1;

static const std::string USB_SOUND_DEVICE      = "USB";
static const std::string BUILD_IN_SOUND_DEVICE = "HDA";

AbleDiagnosticSoundWidget::AbleDiagnosticSoundWidget () : BasicDiagnosticWidget ( true, true )
    , alignment(0.5, 0.5, 0.9, 0.6)
    , soundTestOption(&buttonSoundTest)
{
    progressWidget = FactoryMethodWidget::createProgressWidget();
    progressWidget->setImage("ui/speaker.png");

    GeneralDiagnosisConfiguration & config ( SAESGUIConfigurationManager::getInstance()->getGeneralDiagnosisConfiguration() );

    frame.set_shadow_type(Gtk::SHADOW_NONE);
    frame.set_label_align(0.0,0.5);

    vboxOptions.set_spacing(5);
    vboxOptions.set_homogeneous(false);

    alignment.add(vboxOptions);
    frame.add(alignment);

    ButtonConfiguration buttonConf = config.getButtonConfiguration();
    LabelConfiguration labelText = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->getGeneralDiagnosisConfiguration().getDiagnosisLabelConfiguration() );

    buttonSoundTest.ModifyFontSize ( buttonConf.getFontSize() );
    buttonSoundTest.signal_clicked().connect ( sigc::mem_fun ( *this, &AbleDiagnosticSoundWidget::onPlaySoundTest ) );
    buttonSoundTest.set_size_request ( buttonConf.getButtonWidth(), buttonConf.getButtonHeight() );
    buttonSoundTest.set_no_show_all();
    buttonSoundTest.set_visible ( true );

    soundTestOption.setLabelConfiguration ( labelText );
    soundTestOption.getSignalChange().connect(sigc::mem_fun(*this, &AbleDiagnosticSoundWidget::onOptionSelected));

    vboxOptions.pack_start ( *progressWidget );
    vboxOptions.pack_end ( soundTestOption );

    setWidget ( &frame );
    setAccessibleName ( "AbleDiagnosticSoundWidget" );
}

AbleDiagnosticSoundWidget::~AbleDiagnosticSoundWidget()
{
    remove();

    dispose();
}

void AbleDiagnosticSoundWidget::initialize()
{
    soundTestOption.setTextConteiner ( _ ( "Sound test" ) );
    soundTestOption.setTextOptions ( _ ( "Does the sound test play correctly?" ), _ ( "yes" ), _ ( "no" ) );
    soundTestOption.setSensitive(false, false);
    buttonSoundTest.set_label ( _ ( "Sound test" ) );
    soundTestOption.clearOption();
    setOkButtonSensitive(false);
}

void AbleDiagnosticSoundWidget::dispose()
{
    buttonSoundTest.set_sensitive ( false );

    if (progressWidget != NULL)
    {
        delete progressWidget;
        progressWidget = NULL;
    }
}

bool AbleDiagnosticSoundWidget::getSoundTestIsCorrect()
{
    return soundTestOption.getOption();
}

sigc::signal<void> AbleDiagnosticSoundWidget::getSignalOk()
{
    return signalOkDetected;
}

sigc::signal<void> AbleDiagnosticSoundWidget::getSignalCancel()
{
    return signalCancelDetected;
}

void AbleDiagnosticSoundWidget::onOptionSelected()
{
    setOkButtonSensitive(true);
}

Audio AbleDiagnosticSoundWidget::buildSampleAudio()
{
    Audio audio;
    audio.setSamples((const char *)sampleAudio, sizeof(sampleAudio));
    audio.setBitsPerSample(BITS_PER_SAMPLE);
    audio.setSamplesPerSecond(SAMPLES_PER_SECOND);
    audio.setChannels(CHANNELS);

    return audio;
}

void AbleDiagnosticSoundWidget::onPlaySoundTest ()
{
    Audio audio = buildSampleAudio();

    AudioVotingCheckUtility audioVotingCheckUtility;
    std::string playbackDevice = "";

    if ((audioVotingCheckUtility.ableIsConnected(true)) &&
        (audioVotingCheckUtility.getAbledConnected() == AudioVotingCheckUtility::ABLED20))
    {
        MixerControl mixer(USB_SOUND_DEVICE);
        playbackDevice = mixer.getPlaybackDeviceName();
    }
    else
    {
        MixerControl mixer(BUILD_IN_SOUND_DEVICE);
        playbackDevice = mixer.getPlaybackDeviceName();
    }

    AudioPlayer player(playbackDevice);

    try
    {
        player.play(audio);
    }
    catch (AudioException &audioException)
    {
        const std::string exceptionMessage = Glib::ustring::compose( _( "Audio exception caught when trying to play sample audio: %1" ), audioException.getMessage() );
        logger->Debug( exceptionMessage );
    }

    soundTestOption.setSensitive(true, true);
}
