/**
 * @file DetachedSignature.cxx
 * @brief Body class detached signature
 * @date 08/02/2013
 * @author Yadickson Soto
 */

#include "DetachedSignature.hxx"
#include <System/XML/DomXml.hxx>
#include <System/XML/ExtractCertificate.hxx>
#include "OpenSSLCertificate.hxx"
#include "URILocalResolver.hxx"

#include <xsec/framework/XSECProvider.hpp>
#include <xsec/framework/XSECException.hpp>
#include <xsec/enc/XSECCryptoException.hpp>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <sstream>

#ifdef HAVE_CONFIG_H
#include "../../config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::System::Security;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;

#include <iostream>

ISMTTLog * DetachedSignature::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.DetachedSignature", GETTEXT_PACKAGE );

DetachedSignature::DetachedSignature()
{

}

DetachedSignature::~DetachedSignature()
{

}

bool DetachedSignature::verifyBuffer ( const SafeBuffer & bufferIn, const std::string & basePath )
{
    bool validate = false;

    std::string baseURI = "file://";
    baseURI += basePath;
    baseURI += "/";

    Smartmatic::System::XML::DomXml domXml;
    xercesc::DOMDocument * dom = domXml.bufferToDom ( bufferIn );

    xercesc::DOMNode * resourceEmpty = dom->getDocumentElement();

    resourceEmpty = domXml.getNode ( resourceEmpty, "Signature" );
    resourceEmpty = domXml.getNode ( resourceEmpty, "SignedInfo" );
    resourceEmpty = domXml.getNode ( resourceEmpty, "Reference" );

    if ( resourceEmpty != NULL )
    {
        XMLCh * param = XMLString::transcode ( "URI" );
        xercesc::DOMElement * element = ( xercesc::DOMElement * ) resourceEmpty;
        char * pathString = XMLString::transcode ( element->getAttribute ( param ) );
        std::string path = pathString;
        XSEC_RELEASE_XMLCH ( param );
        XMLString::release ( &pathString );

        if ( path.empty() )
        {
            logger->Debug ( "Resources empty" );
            return true;
        }
    }

    XSECProvider prov;
    DSIGSignature * sig = NULL;

    try
    {
        sig = prov.newSignatureFromDOM ( dom );

        URILocalResolver resolver ( baseURI );

        Smartmatic::System::SafeBuffer base64Cert;
        Smartmatic::System::XML::ExtractCertificate extractCertificate ( bufferIn );

        if ( extractCertificate.getBase64Certificate ( base64Cert ) )
        {
            sig->setURIResolver ( &resolver );
            sig->load();

            OpenSSLCertificate cert ( base64Cert );
            OpenSSLCryptoX509 * key = cert.getX509();

            sig->setSigningKey ( key->clonePublicKey() );

            validate = sig->verify();

            if ( ! validate )
            {
                char * msg = XMLString::transcode ( sig->getErrMsgs() );
                logger->Debug ( "ERROR - " + std::string ( msg ) );
                XMLString::release ( &msg );
            }
        }
        else
        {
            logger->Debug ( "X509 not found" );
        }
    }
    catch ( XSECException & ex )
    {
        char * msg = XMLString::transcode ( ex.getMsg() );
        std::stringstream ss;
        ss << "FATAL - " << msg;
        logger->Debug ( ss.str() );
        XMLString::release ( &msg );
    }
    catch ( ... )
    {
        logger->Debug ( "FATAL - error " + std::string ( __func__ ) );
    }

    if ( sig )
    {
        prov.releaseSignature ( sig );
    }

    return validate;
}
