/**
 * @file XmlSigned.hxx
 * @brief Header class for Xml Signed
 * @author	Yadickson Soto
 * @date	09-02-2013
 */

#ifndef __XML_SIGNED__H__
#define __XML_SIGNED__H__

#include <System/SafeBuffer.h>
#include <xsec/enc/OpenSSL/OpenSSLCryptoX509.hpp>
#include <xsec/enc/XSECCryptoKey.hpp>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            /**
             * @class	XmlSigned
             * @brief	Class for xml signed manager.
             * @author	Yadickson Soto
             * @date	09-02-2013
             */
            class XmlSigned
            {
                public:

                    /**
                     * @fn	XmlSigned ( const Smartmatic::System::SafeBuffer & buffer );
                     * @brief	Class constructor.
                     * @param buffer data buffer xml
                     * @author	Yadickson Soto
                     * @date	09-02-2013
                     */
                    explicit XmlSigned ( const Smartmatic::System::SafeBuffer & buffer );

                    /**
                     * @fn	virtual ~XmlSigned();
                     * @brief	Class destructor.
                     * @author	Yadickson Soto
                     * @date	09-02-2013
                     */
                    virtual ~XmlSigned();

                    /**
                     * @fn	bool signBuffer ( EVP_PKEY * primaryKey, Smartmatic::System::SafeBuffer & result, X509 * cert = NULL, hashMethod hm = HASH_SHA256 );
                     * @brief	method to Sign with a certificate.
                     * @author	Yadickson Soto
                     * @date	09-02-2013
                     * @param	primaryKey primary key
                     * @param 	result result buffer
                     * @param	cert certificate
                     * @param 	hm hash method
                     * @return	true if signed buffer xml
                     */
                    bool signBuffer ( EVP_PKEY * privateKey, Smartmatic::System::SafeBuffer & result, X509 * cert = NULL, hashMethod hm = HASH_SHA256 );

                    /**
                     * @fn	bool verifyBuffer ( X509 * cert = NULL );
                     * @brief	method to validate signed xml with certificate xml
                     * @author	Yadickson Soto
                     * @date	09-02-2013
                     * @param	cert certificate
                     * @return	true if valid buffer xml
                     */
                    bool verifyBuffer ( X509 * cert = NULL );

                private:

                    const Smartmatic::System::SafeBuffer & buffer; ///> buffer data
                    static Smartmatic::Log::ISMTTLog * logger; ///<logger
                    XSECCryptoKey * getKey ( EVP_PKEY * privateKey, signatureMethod & sigMethod );
                    void setKey ( void * sig, OpenSSLCryptoX509 * x509 );
            };
        }
    }
}

#endif /*__XML_SIGNED__H__*/
