#ifndef TESTS_MANAGER_HXX
#define TESTS_MANAGER_HXX

#include <string>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/dom/DOM.hpp>
#include "MockElectoralConfiguration.hxx"
#include "MockSaesFunctionalityConfigurationManager.hxx"
#include <Voting/PBReports/vote-results-schema.hxx>

class TestsManager
{
    public:

        explicit TestsManager();

        ~TestsManager();

    public:

        void initializeElectoralConfiguration();

        void initializeVotes();

        void initializeVoteResults();

        bool getElectionFileFound();

        bool getVotesFileFound();

        bool getVoteResultsFileFound();

        void runConformityTests(const bool runIndividualVoteTests, const bool runVoteResultsTests);

    private:

        void initialize();

        void parseSettingsFile();

        void parseElectionFile();

        void parseVotesFile();

        void parseVoteResultsFile();

        void checkCurrentContest(const xercesc::DOMNode * node);

        void checkCurrentVote(const xercesc::DOMNode * node);

        void checkCurrentSelection(const xercesc::DOMNode * node);

        void checkCurrentEligibleOption(const xercesc::DOMNode * node, const std::string &contestCode);

        void checkCurrentBallotOption(
            const xercesc::DOMNode * node,
            const std::string &contestCode,
            const std::string &eligibleOptionCode,
            const std::string &eligibleOptionParentCode);

        void checkSelectionBallotOption(const xercesc::DOMNode * node, const std::string &contestCode);

        void checkCurrentParty(
            const xercesc::DOMNode * node,
            const std::string &contestCode,
            const std::string &eligibleOptionCode,
            const std::string &eligibleOptionParentCode,
            const std::string &ballotOptionCode,
            const std::string &ballotOptionCategory);

        void checkVoteResults(const xercesc::DOMNode * node);

        void checkContestResults(const xercesc::DOMNode * node, const size_t totalVotes);

        void checkPartyResults(const xercesc::DOMNode * node, const size_t totalVotes, const std::string &contestCode);

        void checkCandidateResult(const xercesc::DOMNode * node);

        void checkSubstituteResult(const xercesc::DOMNode * node);

        void checkCurrentLogicalVoterGroup(const xercesc::DOMNode * node);

        void checkCurrentContestGroup(const xercesc::DOMNode * node);

        void checkCurrentContestCode(const xercesc::DOMNode * node, const std::string &contestGroupCode);

        void setEligibleOptionsByContestParty(
            Smartmatic::SAES::Voting::Election::Eligible_option *eligibleOption,
            const std::string &contestCode,
            const std::string &partyCode);

        void setEligibleOptionsByBallotOptions(
            Smartmatic::SAES::Voting::Election::Eligible_option *eligibleOption,
            const std::string &contestCode);

        void setPartyListByContest(
            const std::string &contestCode,
            const std::string &partyCode,
            const std::string &partyName);

        void setPartyCodeByBallotOptionCode(const std::string &ballotOptionCode, const std::string &partyCode);

        void iterateOverElectionFileContests(
            const xercesc::DOMNode * node,
            const std::string &contestCode = "",
            const std::string &eligibleOptionCode = "",
            const std::string &eligibleOptionParentCode = "",
            const std::string &ballotOptionCode = "",
            const std::string &ballotOptionCategory = "");

        void iterateOverElectionFileLogicalVoterGroups(
            const xercesc::DOMNode * node,
            const std::string &logicalVoterGroupCode = "",
            const std::string &logicalVoterGroupContestGroupCode = "");

        void iterateOverElectionFileContestGroups(
            const xercesc::DOMNode * node,
            const std::string &contestGroupCode = "",
            const std::string &contestCode = "");

        void iterateOverVotesFile(const xercesc::DOMNode * node, const std::string &contestCode = "");

        void iterateOverVoteResultsFile(
            const xercesc::DOMNode * node,
            const size_t totalVotes = 0,
            const std::string &contestCode = "");

    private:

        std::string m_electionFilePath;
        std::string m_votesFilePath;
        std::string m_voteResultsFilePath;

        bool m_electionFileFound;
        bool m_votesFileFound;
        bool m_voteResultsFileFound;

        xercesc::XercesDOMParser* m_electionXml;
        xercesc::XercesDOMParser* m_votesXml;
        xercesc::XercesDOMParser* m_voteResultsXml;

        std::map< std::string, std::list< Smartmatic::SAES::Voting::Election::Eligible_option* > > m_partyListMap;
        std::map< std::string, std::list< Smartmatic::SAES::Voting::Election::Eligible_option* > > m_partyPrincipalMap;
        std::map< std::string, std::list< Smartmatic::SAES::Voting::Election::Eligible_option* > > m_partyStandInMap;

        std::map< std::string, std::map< std::string, std::list<Smartmatic::SAES::Voting::Election::Eligible_option*> > > m_mapEligibleOptionsListByContestParty;
        std::map< std::string, std::map< std::string, std::list<Smartmatic::SAES::Voting::Election::Eligible_option*> > > m_mapEligibleOptionsPrincipalByContestParty;
        std::map< std::string, std::map< std::string, std::list<Smartmatic::SAES::Voting::Election::Eligible_option*> > > m_mapEligibleOptionsStandInByContestParty;

        std::map < std::string, std::map< std::string, Smartmatic::SAES::Voting::Election::Eligible_option* > > m_eligibleOptionByBallotOptionCodeMap;
        std::map < std::string, std::list < Smartmatic::SAES::Voting::Election::Party* > > m_contestPartyMap;

        std::map < std::string, std::string > m_partyCodeByBallotOptionCodeMap;
        std::map < std::string, std::map < std::string, std::string > > m_partyTextToPrintMap;
        std::list < std::map < std::string, std::map < std::string, std::string > > > m_partyTextToPrintMapList;

        std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > m_logicalVoterGroupsList;

        std::map < std::string, Smartmatic::SAES::Voting::Election::Contest_group * > m_contestGroupByCodeMap;
        std::list < std::string > m_contestCodesList;

        MockElectoralConfiguration m_mockElectoralConfiguration;
        MockSaesFunctionalityConfigurationManager m_configurationManager;

        Smartmatic::SAES::Voting::Vote::Vote * m_vote;
        std::list< Smartmatic::SAES::Voting::Vote::Vote* > m_votesList;
        Smartmatic::SAES::Voting::Vote::Selection::BSequence m_ballotOptionsSequence;
        Smartmatic::SAES::Voting::Vote::Selection m_selection;
        Smartmatic::SAES::Voting::Vote::Vote::SSequence m_selectionsSequence;

        size_t m_totalVotes;
        Smartmatic::SAES::Voting::PBReports::VoteResults m_voteResults;
        Smartmatic::SAES::Voting::PBReports::VoteResults::ContestResultsType m_contestResults;
        Smartmatic::SAES::Voting::PBReports::VoteResults::ContestResultsSequence m_contestsSequence;
        Smartmatic::SAES::Voting::PBReports::PartyResults m_partyResults;
        Smartmatic::SAES::Voting::PBReports::ContestResults::PartyResultsSequence m_partiesSequence;
        Smartmatic::SAES::Voting::PBReports::ItemsResults::CandidateSequence m_candidatesSequence;
        Smartmatic::SAES::Voting::PBReports::ItemResult m_candidateResult;
        Smartmatic::SAES::Voting::PBReports::ItemsResults m_candidatesResults;
        Smartmatic::SAES::Voting::PBReports::ItemsResults::CandidateSequence m_substitutesSequence;
        Smartmatic::SAES::Voting::PBReports::ItemResult m_substituteResult;
        Smartmatic::SAES::Voting::PBReports::ItemsResults m_substitutesResults;
};

#endif /* TESTS_MANAGER_HXX */
