/**
 * AsymmetricKeyManager.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.security.asymmetric;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.security.InvalidAlgorithmParameterException;
import java.security.Key;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.List;

import javax.crypto.SecretKey;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.security.io.Base64Encoder;
import com.smartmatic.waffle.security.lang.CryptographicProviderStaticException;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * <p>
 * Description: Class used to handle the asymmetric keys that the application
 * needs to use. Through this class the storage of the keys is modular and
 * centralized.
 * </p>
 * 
 * @version 1.0 (Feb 28, 2008)
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public class AsymmetricKeyManager {

	/**
	 * <p>
	 * Field that define PKCS12 key store type
	 * </p>
	 */

	public final static String KEYSTORE_PKCS12 = "pkcs12";

	/**
	 * <p>
	 * Field that contains the standard parameters used to store the keys
	 * encrypted in a repository
	 * </p>
	 */
	private final static String KEYALG = "RSA";

	/**
	 * <p>
	 * Field that contains the standard parameters bit length used to store the
	 * keys encrypted in a repository
	 * </p>
	 */

	private final static int KEYLENGTH = 2048;

	/**
	 * <p>
	 * Field that contains the standart that is used to store the keys encrypted
	 * in a repository
	 * </p>
	 */

	public final static String KEYSTORE_JKS = "JKS";

	/**
	 * <p>
	 * Method that allows the obtention of a private key contained in a
	 * keystore.
	 * </p>
	 * 
	 * @param inputStream
	 *            InputStream The inputstream where the keystore is located
	 * @param keyStorePassword
	 *            String The key to access into the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @param keyPassword
	 *            String password used to encrypt the key inside the keystore
	 * @param keystoreFormat
	 *            The format of the keystore
	 * @return PrivateKey The private key found in the keystore
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */
	public static PrivateKey getPrivateKey(InputStream inputStream,
			String keyStorePassword, String alias, String keyPassword,
			String keystoreFormat) throws CryptographicStaticException,
			CryptographicRuntimeException {
		try {
			KeyStore keyStore = KeyStore.getInstance(keystoreFormat);
			keyStore.load(inputStream, keyStorePassword.toCharArray());
			Key key = keyStore.getKey(alias, keyPassword.toCharArray());

			return (PrivateKey) key;
		} catch (KeyStoreException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.INVALID_KEYSTORE_OR_WRONG_FORMAT,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (CertificateException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.KEYSTORE_CERTIFICATE_PROBLEM,
					e);
		} catch (UnrecoverableKeyException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.UNRECOVERABLE_ASYMMETRIC_KEY,
					e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		}
	}

	/**
	 * <p>
	 * Method that allows the obtention of a private key contained in a
	 * keystore.
	 * </p>
	 * 
	 * @param inputStream
	 *            InputStream The inputstream where the keystore is located
	 * @param keyStorePassword
	 *            String The key to access into the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @param keyPassword
	 *            String password used to encrypt the key inside the keystore
	 * @return PrivateKey The private key found in the keystore
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if occurred a problem reading / writing a resource
	 */

	public static PrivateKey getPrivateKey(InputStream inputStream,
			String keyStorePassword, String alias, String keyPassword)
			throws CryptographicStaticException, CryptographicRuntimeException {
		return getPrivateKey(inputStream, keyStorePassword, alias, keyPassword,
				AsymmetricKeyManager.KEYSTORE_PKCS12);
	}

	/**
	 * <p>
	 * Method that allows the obtention of a public key contained in a keystore.
	 * </p>
	 * 
	 * @param inputStream
	 *            InputStream The inputstream where the keystore is located
	 * @param keyStorePassword
	 *            String The key to access into the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @param keyPassword
	 *            String password used to encrypt the key inside the keystore
	 * @param keystoreFormat
	 *            The format of the keystore
	 * @return PublicKey The public key found in the keystore
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */

	public static PublicKey getPublicKey(InputStream inputStream,
			String keyStorePassword, String alias, String keyPassword,
			String keystoreFormat) throws CryptographicStaticException,
			CryptographicRuntimeException {
		try {
			KeyStore keyStore = KeyStore.getInstance(keystoreFormat);
			keyStore.load(inputStream, keyStorePassword.toCharArray());

			Certificate cert = keyStore.getCertificate(alias);

			Key key = cert.getPublicKey();

			return (PublicKey) key;
		} catch (KeyStoreException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.INVALID_KEYSTORE_OR_WRONG_FORMAT,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (CertificateException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.KEYSTORE_CERTIFICATE_PROBLEM,
					e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		}
	}

	/**
	 * <p>
	 * Method that allows the obtention of a public key contained in a keystore.
	 * </p>
	 * 
	 * @param inputStream
	 *            InputStream The inputstream where the keystore is located
	 * @param keyStorePassword
	 *            String The key to access into the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @param keyPassword
	 *            String password used to encrypt the key inside the keystore
	 * @return PublicKey The public key found in the keystore
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */

	public static PublicKey getPublicKey(InputStream inputStream,
			String keyStorePassword, String alias, String keyPassword)
			throws CryptographicStaticException, CryptographicRuntimeException {
		return getPublicKey(inputStream, keyStorePassword, alias, keyPassword,
				AsymmetricKeyManager.KEYSTORE_PKCS12);
	}

	/**
	 * <p>
	 * Method that allows to create a keystore with the default type PKCS12
	 * </p>
	 * 
	 * @param key
	 *            PrivateKey The private key wich is going to contain the
	 *            keystore
	 * @param alias
	 *            String The alias used to create the keystore
	 * @param keyStorePassword
	 *            String The password used to secure the keystore
	 * @param keyPassword
	 *            String The password used to secure the key in the keystore. If
	 *            the key password is null, the method set as keyPassword the
	 *            keyStorePassword.
	 * @param certChain
	 *            Certificate[] An array of certificates used to create the
	 *            keystore
	 * @return Keystore The newly keystore
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */
	public static KeyStore createKeystore(PrivateKey key, String alias,
			String keyStorePassword, String keyPassword, Certificate[] certChain)
			throws CryptographicStaticException, CryptographicRuntimeException {
		return createKeystore(key, alias, keyStorePassword, keyPassword,
				certChain, AsymmetricKeyManager.KEYSTORE_PKCS12);
	}

	/**
	 * <p>
	 * Method that allows to create a keystore
	 * </p>
	 * 
	 * @param key
	 *            PrivateKey The private key wich is going to contain the
	 *            keystore
	 * @param alias
	 *            String The alias used to create the keystore
	 * @param keyStorePassword
	 *            String The password used to secure the keystore
	 * @param keyPassword
	 *            String The password used to secure the key in the keystore. If
	 *            the key password is null, the method set as keyPassword the
	 *            keyStorePassword.
	 * @param certChain
	 *            Certificate[] An array of certificates used to create the
	 *            keystore
	 * @param keyStoreType
	 *            String The type of the keystore
	 * @return Keystore The newly keystore
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */
	public static KeyStore createKeystore(PrivateKey key, String alias,
			String keyStorePassword, String keyPassword,
			Certificate[] certChain, String keyStoreType)
			throws CryptographicStaticException, CryptographicRuntimeException {
		try {
			KeyStore keyStore = KeyStore.getInstance(keyStoreType);
			keyStore.load(null, keyStorePassword.toCharArray());
			if (keyPassword != null) {
				keyStore.setKeyEntry(alias, key, keyPassword.toCharArray(),
						certChain);
			} else {
				keyStore.setKeyEntry(alias, key,
						keyStorePassword.toCharArray(), certChain);
			}

			return keyStore;
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);

		} catch (Exception e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);

		}
	}

	/**
	 * <p>
	 * Method that allows to create a pkcs12keystore with no key
	 * </p>
	 * 
	 * @param certChain
	 *            Certificate[] An array of certificates used to create the
	 *            keystore
	 * @param keytore
	 *            the keytore
	 * @param pass
	 *            the pass
	 * @return the key store
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */
	public static KeyStore createPkcs12Keystore(List<Certificate> certChain,
			String keytore, char[] pass) throws CryptographicStaticException,
			CryptographicRuntimeException {
		KeyStore keyStore;
		java.io.FileInputStream fis = null;
		try {
			keyStore = KeyStore.getInstance(
					AsymmetricKeyManager.KEYSTORE_PKCS12, "BC");
			File key = new File(keytore);
			// key.createNewFile();
			if (key.exists()) {
				fis = new java.io.FileInputStream(keytore);
				keyStore.load(fis, pass);
			} else {
				keyStore.load(null, pass);
			}
			// cambiar el "alias" por algo propio del crt.
			for (Certificate cert : certChain) {
				keyStore.setCertificateEntry(cert.toString(), cert);
			}

			return keyStore;
		} catch (KeyStoreException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (CertificateException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.KEYSTORE_CERTIFICATE_PROBLEM,
					e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		} catch (NoSuchProviderException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		}
	}

	/**
	 * <p>
	 * Method that generate a new KeyPair Key of length KEYLENGTH and algorithm
	 * KEYALG.
	 * </p>
	 * 
	 * @return KeyPair The newly keypair generated
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */

	public static KeyPair generateKeyPair()
			throws CryptographicStaticException, CryptographicRuntimeException {
		return generateKeyPair(AsymmetricKeyManager.KEYALG,
				AsymmetricKeyManager.KEYLENGTH);
	}

	/**
	 * /**
	 * <p>
	 * Method that generate a new KeyPair of length defined by the caller. This
	 * method use the default algorithm KEYALG
	 * </p>
	 * 
	 * @param keyLength
	 *            the keysize. This is an algorithm-specific metric, such as
	 *            modulus length, specified in number of bits.
	 * @return KeyPair The newly keypair generated
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 */

	public static KeyPair generateKeyPair(int keyLength)
			throws CryptographicRuntimeException, CryptographicStaticException {
		return generateKeyPair(AsymmetricKeyManager.KEYALG, keyLength);
	}

	/**
	 * <p>
	 * Method that generate a new KeyPair of length and algorithm defined by the
	 * caller.
	 * </p>
	 * 
	 * @param alg
	 *            the standard string name of the algorithm. See Appendix A in
	 *            the Java Cryptography Architecture API Specification &
	 *            Reference for information about standard algorithm names.
	 * @param keyLength
	 *            the keysize. This is an algorithm-specific metric, such as
	 *            modulus length, specified in number of bits.
	 * @return KeyPair The newly keypair generated
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 */

	public static KeyPair generateKeyPair(String alg, int keyLength)
			throws CryptographicRuntimeException, CryptographicStaticException {
		try {
			KeyPairGenerator keyGenerator = KeyPairGenerator.getInstance(alg);

			keyGenerator.initialize(keyLength);

			KeyPair keyPair = keyGenerator.generateKeyPair();

			return keyPair;
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		}
	}

	/**
	 * <p>
	 * Method that generate a new KeyPair of length and algorithm defined by the
	 * caller.
	 * </p>
	 * 
	 * @param alg
	 *            the standard string name of the algorithm. See Appendix A in
	 *            the Java Cryptography Architecture API Specification &
	 *            Reference for information about standard algorithm names.
	 * @param provider
	 *            the provider
	 * @param keyLength
	 *            the keysize. This is an algorithm-specific metric, such as
	 *            modulus length, specified in number of bits.
	 * @return KeyPair The newly keypair generated
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws NoSuchProviderException
	 *             the no such provider exception
	 * @throws InvalidAlgorithmParameterException
	 *             the invalid algorithm parameter exception
	 */

	public static KeyPair generateKeyPair(String alg, String provider,
			int keyLength) throws CryptographicRuntimeException,
			CryptographicStaticException, NoSuchProviderException,
			InvalidAlgorithmParameterException {
		try {

			KeyPairGenerator keyGenerator = KeyPairGenerator.getInstance(alg,
					provider);

			keyGenerator.initialize(keyLength);

			KeyPair keyPair = keyGenerator.generateKeyPair();

			return keyPair;

		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		}
	}

	/**
	 * <p>
	 * Method that allows the obtention of a certificate contained in a
	 * keystore.
	 * </p>
	 * 
	 * @param inputStream
	 *            InputStream The inputstream where the keystore is located
	 * @param keyStorePassword
	 *            String The key to access into the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @param keystoreFormat
	 *            The format of the keystore
	 * @return X509Certificate The certificate found in the keystore
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */

	public static X509Certificate getCertificate(InputStream inputStream,
			String keyStorePassword, String alias, String keystoreFormat)
			throws CryptographicStaticException, CryptographicRuntimeException {
		try {
			KeyStore keyStore = KeyStore.getInstance(keystoreFormat);
			keyStore.load(inputStream, keyStorePassword.toCharArray());

			X509Certificate cert = (X509Certificate) keyStore
					.getCertificate(alias);
			return cert;
		} catch (KeyStoreException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.INVALID_KEYSTORE_OR_WRONG_FORMAT,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (CertificateException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.KEYSTORE_CERTIFICATE_PROBLEM,
					e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		}
	}

	/**
	 * <p>
	 * Method that obtain a certificate from the input stream that holds it.
	 * This method is used to verfy if a file contains a valid X.509 certificate
	 * </p>
	 * 
	 * @param stream
	 *            InputStream The input stream where the certificate is located
	 * @return Certificate The certificate loaded from the input stream
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */

	public static X509Certificate getCertificate(InputStream stream)
			throws CryptographicRuntimeException {
		try {
			CertificateFactory cf = CertificateFactory.getInstance("X.509");
			X509Certificate certificate = (X509Certificate) cf
					.generateCertificate(stream);
			return certificate;
		} catch (CertificateException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_CERTIFICATE_FORMAT,
					e);
		}
	}

	/**
	 * <p>
	 * Method that allows the obtention of a certificate contained in a
	 * keystore.
	 * </p>
	 * 
	 * @param inputStream
	 *            InputStream The inputstream where the keystore is located
	 * @param keyStorePassword
	 *            String The key to access into the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @return X509Certificate The certificate found in the keystore
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */

	public static X509Certificate getCertificate(InputStream inputStream,
			String keyStorePassword, String alias)
			throws CryptographicStaticException, CryptographicRuntimeException {
		return getCertificate(inputStream, keyStorePassword, alias,
				AsymmetricKeyManager.KEYSTORE_PKCS12);
	}

	/**
	 * Method that retrieves the certificate chain contained in a keystore.
	 * 
	 * @param inputStream
	 *            InputStream The inputstream where the keystore is located
	 * @param keyStorePassword
	 *            String The key to access into the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @return The certificate chain contained in the given keystore
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */
	public static Certificate[] getCertificateChain(InputStream inputStream,
			String keyStorePassword, String alias)
			throws CryptographicStaticException, CryptographicRuntimeException {
		return getCertificateChain(inputStream, keyStorePassword, alias, "");
	}

	/**
	 * Method that retrieves the certificate chain contained in a keystore.
	 * 
	 * @param inputStream
	 *            InputStream The inputstream where the keystore is located
	 * @param keyStorePassword
	 *            String The key to access into the keystore
	 * @param alias
	 *            String alias used to mark the key introduced in the keystore
	 * @param keyStorePath
	 *            The Path of the keystore, used in the logs.
	 * @return The certificate chain contained in the given keystore
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */
	public static Certificate[] getCertificateChain(InputStream inputStream,
			String keyStorePassword, String alias, String keyStorePath)
			throws CryptographicStaticException, CryptographicRuntimeException {
		try {
			KeyStore keyStore = KeyStore
					.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);
			keyStore.load(inputStream, keyStorePassword.toCharArray());
			X509Certificate certificate = (X509Certificate) keyStore
					.getCertificate(alias);

			if (certificate != null) {
				// certificate.checkValidity();

				return keyStore.getCertificateChain(alias);
			} else {
				return null;
			}
		} catch (KeyStoreException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.INVALID_KEYSTORE_OR_WRONG_FORMAT,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (CertificateException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.KEYSTORE_CERTIFICATE_PROBLEM,
					e);
		} catch (FileNotFoundException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		}
	}

	/**
	 * Method that obtains and encodes the certificate in his stantar format.
	 * 
	 * @param certificate
	 *            The certificate to encode
	 * @return The Certificate base 64 representation
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */

	public static String encodeCertificate(
			java.security.cert.Certificate certificate)
			throws CryptographicRuntimeException {
		return encodeCertificate(certificate,
				System.getProperty("line.separator"));
	}

	/**
	 * Method that obtains and encodes the certificate in his stantar format.
	 * 
	 * @param certificate
	 *            The certificate to encode
	 * @param lineSeparator
	 *            the line separator
	 * @return The Certificate base 64 representation
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */

	public static String encodeCertificate(
			java.security.cert.Certificate certificate, String lineSeparator)
			throws CryptographicRuntimeException {
		try {
			StringBuffer certB64 = new StringBuffer(
					"-----BEGIN CERTIFICATE-----\n");

			String certificateEncoded = Base64Encoder.encode(certificate
					.getEncoded());

			// Checks the line separator
			String systemLineSeparator = System.getProperty("line.separator");
			if (!systemLineSeparator.equals(lineSeparator)) {
				certificateEncoded = StringUtils.replace(certificateEncoded,
						systemLineSeparator, lineSeparator);

			}
			certB64.append(certificateEncoded);
			certB64.append("\n-----END CERTIFICATE-----");

			return certB64.toString();
		} catch (CertificateEncodingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_CERTIFICATE_FORMAT,
					e);
		}
	}

	/**
	 * <p>
	 * Method that obtain using the constants defined in this class the private
	 * key that the reception MODULE have to use to be able to decrypt the
	 * information sent by the voting machines in the transmission phase
	 * </p>
	 * 
	 * @param privateKeyPath
	 *            the private key path
	 * @param asymmetricKeyStorePassword
	 *            the asymmetric key store password
	 * @param asymmetricAlias
	 *            the asymmetric alias
	 * @param asymmetricKeyPassword
	 *            the asymmetric key password
	 * @return PrivateKey The private key used in the handshake phase of the
	 *         protocol of comunication with the voting machines.
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */

	public static PrivateKey getPrivateKey(String privateKeyPath,
			String asymmetricKeyStorePassword, String asymmetricAlias,
			String asymmetricKeyPassword) throws CryptographicStaticException,
			CryptographicRuntimeException {
		FileInputStream fileStream = null;
		try {
			fileStream = new FileInputStream(privateKeyPath);
			return getPrivateKey(fileStream, asymmetricKeyStorePassword,
					asymmetricAlias, asymmetricKeyPassword, privateKeyPath);
		} catch (FileNotFoundException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		} finally {
			try {
				if (fileStream != null) {
					fileStream.close();
				}
			} catch (IOException e) {
				throw new CryptographicRuntimeException(
						CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM,
						e);
			}
		}
	}

	/**
	 * <p>
	 * Method that obtain using the constants defined in this class the private
	 * key that the reception MODULE have to use to be able to decrypt the
	 * information sent by the voting machines in the handshake phase
	 * </p>
	 * 
	 * @param privateHandShakeKeyPath
	 *            the private hand shake key path
	 * @param privateHandShakeKeyStorePassword
	 *            the private hand shake key store password
	 * @param privateHandShakeKeyAlias
	 *            the private hand shake key alias
	 * @param privateHandShakeKeyPassword
	 *            the private hand shake key password
	 * @return PrivateKey The private key used in the handshake phase of the
	 *         protocol of comunication with the voting machines.
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */

	public static PrivateKey getHandShakePrivateKey(
			String privateHandShakeKeyPath,
			String privateHandShakeKeyStorePassword,
			String privateHandShakeKeyAlias, String privateHandShakeKeyPassword)
			throws CryptographicStaticException, CryptographicRuntimeException {
		FileInputStream fileStream = null;
		try {
			fileStream = new FileInputStream(privateHandShakeKeyPath);
			return getPrivateKey(fileStream, privateHandShakeKeyStorePassword,
					privateHandShakeKeyAlias, privateHandShakeKeyPassword,
					privateHandShakeKeyPath);
		} catch (FileNotFoundException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		} finally {
			try {
				if (fileStream != null) {
					fileStream.close();
				}
			} catch (IOException e) {
				throw new CryptographicRuntimeException(
						CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM,
						e);
			}
		}
	}

	/**
	 * Method that create a keystore given a secret key, the alias under the key
	 * is going to be stored and the passwoord of the keystore.
	 * 
	 * @param secretKey
	 *            The key that should be persisted in the keystore
	 * @param alias
	 *            Teh alias that will be used to store the key in the keystore
	 * @param keyStorePassword
	 *            The password of the keystore
	 * @return A output stream that contains the keystore
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */

	public static KeyStore createKeystore(SecretKey secretKey, String alias,
			String keyStorePassword) throws CryptographicStaticException,
			CryptographicRuntimeException {
		try {
			KeyStore keyStore = KeyStore
					.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);
			keyStore.load(null, keyStorePassword.toCharArray());
			keyStore.setKeyEntry(alias, secretKey, null, null);
			return keyStore;
		} catch (KeyStoreException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.INVALID_KEYSTORE_OR_WRONG_FORMAT,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (CertificateException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.KEYSTORE_CERTIFICATE_PROBLEM,
					e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		}
	}

	/**
	 * Method that allows to create a keystore and returns the input stream that
	 * contains the information inside the keystore.
	 * 
	 * @param key
	 *            the key
	 * @param keyStorePassword
	 *            the key store password
	 * @param keyPassword
	 *            the key password
	 * @param certChain
	 *            the cert chain
	 * @param alias
	 *            the alias
	 * @return the input stream
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */

	public static InputStream createKeystore(PrivateKey key,
			String keyStorePassword, String keyPassword,
			Certificate[] certChain, String alias)
			throws CryptographicStaticException, CryptographicRuntimeException {

		try {
			KeyStore keyStore = KeyStore
					.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);
			keyStore.load(null, keyStorePassword.toCharArray());

			/* Check the values of the parameters */
			if (alias == null) {
				throw new NullPointerException("Alias");
			}

			/* Check the values of the parameters */
			if (key == null) {
				throw new NullPointerException("key");
			}

			/* Check the values of the parameters */
			if (keyPassword == null) {
				throw new NullPointerException("keyPassword");
			}

			keyStore.setKeyEntry(alias, key, keyPassword.toCharArray(),
					certChain);
			ByteArrayOutputStream byteStream = new ByteArrayOutputStream();

			keyStore.store(byteStream, keyStorePassword.toCharArray());

			ByteArrayInputStream inputStream = new ByteArrayInputStream(
					byteStream.toByteArray());
			return inputStream;
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (KeyStoreException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.INVALID_KEYSTORE_OR_WRONG_FORMAT,
					e);
		} catch (CertificateException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.KEYSTORE_CERTIFICATE_PROBLEM,
					e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		}
	}

	/**
	 * Return a Certificate from a keyStore.
	 * 
	 * @param inputStream
	 *            The Stream that represent the keyStore
	 * @param keyStorePassword
	 *            the key store password
	 * @param alias
	 *            The Certificate alias
	 * @param format
	 *            the format
	 * @return a <code>X509Certificate</code>
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 * @see
	 * @see java.security.KeyStore
	 */
	public static Certificate getCertificateByFormatKeystore(
			InputStream inputStream, String keyStorePassword, String alias,
			String format) throws CryptographicStaticException,
			CryptographicRuntimeException {

		try {
			KeyStore keyStore;
			keyStore = KeyStore.getInstance(format);
			keyStore.load(inputStream, keyStorePassword.toCharArray());

			X509Certificate certificate = (X509Certificate) keyStore
					.getCertificate(alias);

			return certificate;
		} catch (KeyStoreException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.INVALID_KEYSTORE_OR_WRONG_FORMAT,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (CertificateException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.KEYSTORE_CERTIFICATE_PROBLEM,
					e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		}

	}

	/**
	 * <p>
	 * Method that obtain a public key from a input stream that contains a X.509
	 * Certificate.
	 * </p>
	 * 
	 * @param stream
	 *            InputStream The input stream where is located the certificate
	 * @return PublicKey The key present in the certificate
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */

	public static PublicKey getPublicKey(InputStream stream)
			throws CryptographicStaticException, CryptographicRuntimeException {
		java.security.cert.Certificate certificate = getCertificate(stream);
		PublicKey key = certificate.getPublicKey();
		return key;
	}

	/**
	 * Method used to check that a pair of keys (public and private) are correct
	 * and each one corresponds to the other.
	 * 
	 * @param pubKey
	 *            the pub key
	 * @param privateKey
	 *            the private key
	 * @return true if the key pair is valid false otherwise
	 */

	public static boolean checkKeyPair(PublicKey pubKey, PrivateKey privateKey) {
		try {
			AsymmetricCipherPool pubCipher = new AsymmetricCipherPool(pubKey);
			AsymmetricCipherPool privateCipher = new AsymmetricCipherPool(
					privateKey);
			String plainData = new Date().toString();
			String dataEnc = pubCipher.encrypt(plainData);
			String dataDec = privateCipher.decrypt(dataEnc);
			return plainData.compareTo(dataDec) == 0;
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * Gets the private key in pkc s12 format.
	 * 
	 * @param inputStream
	 *            the input stream
	 * @param keyStorePassword
	 *            the key store password
	 * @param alias
	 *            the alias
	 * @param keyPassword
	 *            the key password
	 * @return the private key in pkc s12 format
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */
	public static PrivateKey getPrivateKeyInPKCS12Format(
			InputStream inputStream, String keyStorePassword, String alias,
			String keyPassword) throws CryptographicStaticException,
			CryptographicRuntimeException {
		try {
			KeyStore keyStore = KeyStore
					.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);
			keyStore.load(inputStream, keyStorePassword.toCharArray());
			Key key = keyStore.getKey(alias, keyPassword.toCharArray());
			return (PrivateKey) key;
		} catch (KeyStoreException e) {
			throw new CryptographicProviderStaticException(
					CryptographicStaticException.ExceptionInfo.INVALID_KEYSTORE_OR_WRONG_FORMAT,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicProviderStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (CertificateException e) {
			throw new CryptographicProviderStaticException(
					CryptographicStaticException.ExceptionInfo.KEYSTORE_CERTIFICATE_PROBLEM,
					e);
		} catch (UnrecoverableKeyException e) {
			throw new CryptographicProviderStaticException(
					CryptographicStaticException.ExceptionInfo.UNRECOVERABLE_ASYMMETRIC_KEY,
					e);
		} catch (FileNotFoundException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		}
	}

}