/**
 * PoolableAsymmetricCipher.java
 *
 * $Revision: 1.5 $ ($Date: 2011-04-06 23:22:02 $)
 *
 * Copyright (c) 2004 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */

package com.smartmatic.waffle.security.asymmetric;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import org.apache.commons.codec.binary.Base64;

import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * Object that its used to encrypt and decrypt information. This object its a
 * poolable object because the cost of create a asymmetric cipher its expensive
 * so the receiver module its configured to use a predefined set of ciphers.
 * 
 * @version 1.0 (Feb 26, 2008)
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public class PoolableAsymmetricCipher {

	/**
	 * Criptographic Algorithm that will be used in the encryption / decryption
	 * process.
	 */

	private final static String KEYALG = "RSA";

	/**
	 * Extended name of the algorithm that will be used in the encryption /
	 * decryption proccess. The format to define the extended name of the
	 * algorihtm is: Algorithm/Mode/Padding </p>
	 * 
	 * @see <a href=
	 *      "http://java.sun.com/j2se/1.4.2/docs/guide/security/jce/JCERefGuide.html#CipherClass"
	 *      > Java Cryptographic Extension< /a>
	 */

	private final static String ALGORITHM = PoolableAsymmetricCipher.KEYALG
			+ "/ECB/PKCS1Padding";

	/**
	 * <p>
	 * Object that its responsible of the encryption process
	 * </p>
	 */

	private Cipher encryptCipher;

	/**
	 * <p>
	 * Object that its responsible of the decryption process
	 * </p>
	 */

	private Cipher decryptCipher;

	/**
	 * <p>
	 * This constructor creates a PoolableAsymmetricCipher with the public key
	 * received as argument used to encrypt/decrypt the information received by
	 * the server.
	 * </p>
	 * 
	 * @param publicKey
	 *            PublicKey Key used in the encryption/decryption process
	 * @param algorithm
	 *            Detail of the algorithm, padding and feedback mode to use in
	 *            this cipher.
	 * @throws CryptographicStaticException
	 * @throws CryptographicRuntimeException
	 */

	public PoolableAsymmetricCipher(PublicKey publicKey, String algorithm)
			throws CryptographicStaticException, CryptographicRuntimeException {
		try {
			String keyAlgorithm = publicKey.getAlgorithm();
			if (algorithm.indexOf(keyAlgorithm) == -1) {
				throw new CryptographicRuntimeException(
						CryptographicRuntimeException.ExceptionInfo.ASYMMETRIC_KEY_PROBLEM);
			}
			this.encryptCipher = Cipher.getInstance(algorithm);
			this.encryptCipher.init(Cipher.ENCRYPT_MODE, publicKey);

			this.decryptCipher = Cipher.getInstance(algorithm);
			this.decryptCipher.init(Cipher.DECRYPT_MODE, publicKey);
		} catch (CryptographicRuntimeException e) {
			throw e;
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (NoSuchPaddingException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.PADDING_OR_PROVIDER_PROBLEM,
					e);
		} catch (InvalidKeyException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.ASYMMETRIC_KEY_PROBLEM,
					e);
		} catch (Exception e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.UNCATEGORIZED_PROBLEM,
					e);
		}
	}

	/**
	 * <p>
	 * This constructor creates a PoolableAsymmetricCipher with the private key
	 * received as argument used to encrypt/decrypt the information received by
	 * the server.
	 * </p>
	 * 
	 * @param privateKey
	 *            PrivateKey Key used in the encryption/decryption proccess
	 * @param algorithm
	 *            Detail of the algorithm, padding and feedback mode to use in
	 *            this cipher.
	 * @throws CryptographicStaticException
	 * @throws CryptographicRuntimeException
	 */

	public PoolableAsymmetricCipher(PrivateKey privateKey, String algorithm)
			throws CryptographicStaticException, CryptographicRuntimeException {
		try {
			String keyAlgorithm = privateKey.getAlgorithm();
			if (algorithm.indexOf(keyAlgorithm) == -1) {
				throw new CryptographicRuntimeException(
						CryptographicRuntimeException.ExceptionInfo.ASYMMETRIC_KEY_PROBLEM);
			}
			this.encryptCipher = Cipher.getInstance(algorithm);
			this.encryptCipher.init(Cipher.ENCRYPT_MODE, privateKey);

			this.decryptCipher = Cipher.getInstance(algorithm);
			this.decryptCipher.init(Cipher.DECRYPT_MODE, privateKey);
		} catch (CryptographicRuntimeException e) {
			throw e;
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (NoSuchPaddingException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.PADDING_OR_PROVIDER_PROBLEM,
					e);
		} catch (InvalidKeyException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.ASYMMETRIC_KEY_PROBLEM,
					e);
		}
	}

	/**
	 * <p>
	 * This constructor creates a PoolableAsymmetricCipher with the public key
	 * received as argument used to encrypt/decrypt the information received by
	 * the server. This constructor asummes that the algorithm used to
	 * enencrypt/decrypt the information is the one set by default by this
	 * class.
	 * </p>
	 * 
	 * @param publicKey
	 *            PublicKey Key used in the encryption/decryption proccess
	 * @throws CryptographicStaticException
	 * @throws CryptographicRuntimeException
	 */

	public PoolableAsymmetricCipher(PublicKey publicKey)
			throws CryptographicStaticException, CryptographicRuntimeException {
		this(publicKey, PoolableAsymmetricCipher.ALGORITHM);
	}

	/**
	 * <p>
	 * This constructor creates a PoolableAsymmetricCipher with the private key
	 * received as argument used to encrypt/decrypt the information received by
	 * the server. This constructor asummes that the algorithm used to
	 * enencrypt/decrypt the information is the one set by default by this
	 * class.
	 * </p>
	 * 
	 * @param privateKey
	 *            PrivateKey Key used in the encryption/decryption proccess
	 */

	public PoolableAsymmetricCipher(PrivateKey privateKey)
			throws CryptographicStaticException, CryptographicRuntimeException {
		this(privateKey, PoolableAsymmetricCipher.ALGORITHM);
	}

	/**
	 * <p>
	 * Method that decrypts a message encrypted with an asymmetric algorithm
	 * using the key received in the constructor
	 * </p>
	 * 
	 * @param message
	 *            String The message that needs to be decrypted
	 * @return String The plain message.
	 * @throws CryptographicRuntimeException
	 *             if a problem in the decryption process occurs
	 */

	public String decrypt(String message) throws CryptographicRuntimeException {
		try {
			byte[] dataDecoded = Base64.decodeBase64(message.getBytes());
			byte[] dataPlain = this.decryptCipher.doFinal(dataDecoded);
			String plainText = new String(dataPlain);
			return plainText;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.PADDING_PROBLEM,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.ILEGAL_BLOCK_PROBLEM,
					e);
		} catch (Exception e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.UNCATEGORIZED_PROBLEM,
					e);
		}
	}

	/**
	 * <p>
	 * Method that decrypts a message encrypted with an asymmetric algorithm
	 * using the key received in the constructor
	 * </p>
	 * 
	 * @param data
	 *            byte[] The package in binary format
	 * @return byte[] The plain package in binary format.
	 * @throws CryptographicRuntimeException
	 *             if a problem in the decryption process occurs
	 */

	public byte[] decryptBytes(byte[] data)
			throws CryptographicRuntimeException {
		try {
			byte[] dataPlain = this.decryptCipher.doFinal(data);
			return dataPlain;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.PADDING_PROBLEM,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.ILEGAL_BLOCK_PROBLEM,
					e);
		} catch (Exception e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.UNCATEGORIZED_PROBLEM,
					e);
		}
	}

	/**
	 * <p>
	 * Method that encrypts a message with an asymmetric algorithm using the key
	 * received in the constructor
	 * </p>
	 * 
	 * @param message
	 *            String The message that needs to be encrypted
	 * @return String The encrypted message. The result of the encryption is
	 *         always a printable string because after the encryption proccess
	 *         the result is converted into a Base64 String
	 * 
	 * @throws CryptographicRuntimeException
	 *             if a problem in the encryption process occurs
	 */

	public String encrypt(String message) throws CryptographicRuntimeException {
		try {
			byte[] dataEncoded = this.encryptCipher.doFinal(message.getBytes());
			String cipherText = new String(Base64.encodeBase64(dataEncoded,
					false));
			return cipherText;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.PADDING_PROBLEM,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.ILEGAL_BLOCK_PROBLEM,
					e);
		} catch (Exception e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.UNCATEGORIZED_PROBLEM,
					e);
		}
	}

	/**
	 * <p>
	 * Method that encrypts a message with an asymmetric algorithm using the key
	 * received in the constructor
	 * </p>
	 * 
	 * @param message
	 *            byte[] The message in binary format that needs to be encrypted
	 * @return byte[] The encrypted message in binary format.
	 * 
	 * @throws CryptographicRuntimeException
	 *             if a problem in the encryption process occurs
	 */

	public byte[] encryptBytes(byte[] message)
			throws CryptographicRuntimeException {
		try {
			byte[] dataEncoded = this.encryptCipher.doFinal(message);
			return dataEncoded;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.PADDING_PROBLEM,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.ILEGAL_BLOCK_PROBLEM,
					e);
		} catch (Exception e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.UNCATEGORIZED_PROBLEM,
					e);
		}
	}

}
