/*
 * SignGenerator.java
 *
 * $Revision: 1.6 $ ($Date: 2011-04-22 20:46:40 $)
 *
 * Copyright (c) 2004 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */

package com.smartmatic.waffle.security.sign;

import java.io.IOException;
import java.io.Serializable;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dsig.CanonicalizationMethod;
import javax.xml.crypto.dsig.DigestMethod;
import javax.xml.crypto.dsig.Reference;
import javax.xml.crypto.dsig.SignatureMethod;
import javax.xml.crypto.dsig.SignedInfo;
import javax.xml.crypto.dsig.Transform;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.dom.DOMSignContext;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import javax.xml.crypto.dsig.keyinfo.KeyInfoFactory;
import javax.xml.crypto.dsig.keyinfo.X509Data;
import javax.xml.crypto.dsig.spec.C14NMethodParameterSpec;
import javax.xml.crypto.dsig.spec.TransformParameterSpec;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.jcp.xml.dsig.internal.dom.XMLDSigRI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Class used to add digital signature to a xml file using the W3C digital
 * signature recommendation
 * 
 * @author Hector Rodríguez (hector.rodriguez@smartmatic.com)
 */

public class SignGenerator {

	/**
	 * Certificates used in the signing process
	 */

	private X509Certificate[] certificates;

	/**
	 * Private keys used in the signing process
	 */

	private PrivateKey[] privateKeys;

	/**
	 * Xml signature factory
	 */

	private XMLSignatureFactory xmlSignatureFactory;

	/**
	 * Signed info array
	 */

	private SignedInfo[] signedInfoArray;

	/**
	 * Key info array
	 */

	private KeyInfo[] keyInfoArray;

	/**
	 * Algorithm to sign
	 */
	private String signatureAlgorithm;

	/**
	 * Constructor. Creates a new sign generator instance with only a signing
	 * certificate and private key
	 * 
	 * @param cert
	 *            The cert used in the signing process
	 * @param privateKey
	 *            The pirvate keys used in the signing process
	 * @param signatureAlgorithm
	 *            The algorithm to sign
	 * 
	 * @throws NoSuchAlgorithmException
	 *             If a problem in the algorithms used in the signing process
	 *             occurs
	 * @throws InvalidAlgorithmParameterException
	 *             If a problem in one or more of the signing parameters occurs
	 */

	public SignGenerator(X509Certificate cert, PrivateKey privateKeys,
			String signatureAlgorithm) throws NoSuchAlgorithmException,
			InvalidAlgorithmParameterException {
		this(new X509Certificate[] { cert }, new PrivateKey[] { privateKeys },
				signatureAlgorithm);
	}

	/**
	 * Constructor. Creates a new sign generator instance with many signing
	 * certificates and private keys.
	 * 
	 * @param cert
	 *            The cert arrays used in the signing process
	 * @param privateKey
	 *            The pirvate keys arrays used in the signing process
	 * @param signatureAlgorithm
	 *            The algorithm to sign
	 * 
	 * @throws NoSuchAlgorithmException
	 *             If a problem in the algorithms used in the signing process
	 *             occurs
	 * @throws InvalidAlgorithmParameterException
	 *             If a problem in one or more of the signing parameters occurs
	 */
	public SignGenerator(X509Certificate[] cert, PrivateKey[] privateKeys,
			String signatureAlgorithm) throws NoSuchAlgorithmException,
			InvalidAlgorithmParameterException {
		this.certificates = (cert != null) ? cert.clone() : null;
		this.privateKeys = (privateKeys != null) ? privateKeys.clone() : null;
		this.signatureAlgorithm = signatureAlgorithm;
		init();
	}

	/**
	 * Method used to instantiate objects necessary to perform digital signs.
	 * 
	 * @throws NoSuchAlgorithmException
	 * @throws InvalidAlgorithmParameterException
	 */
	private void init() throws NoSuchAlgorithmException,
			InvalidAlgorithmParameterException {

		// Init daemon santuario xml security

		// Create a DOM XMLSignatureFactory that will be used to
		// generate the enveloped signature.

		this.xmlSignatureFactory = XMLSignatureFactory.getInstance("DOM",
				new XMLDSigRI());

		// Create a Reference to the enveloped document (in this case,
		// you are signing the whole document, so a URI of "" signifies
		// that, and also specify the SHA1 digest algorithm and
		// the ENVELOPED Transform.
		Reference ref = this.xmlSignatureFactory.newReference("",
				this.xmlSignatureFactory.newDigestMethod(DigestMethod.SHA256,
						null), Collections
						.singletonList(this.xmlSignatureFactory.newTransform(
								Transform.ENVELOPED,
								(TransformParameterSpec) null)), null, null);

		this.signedInfoArray = new SignedInfo[this.certificates.length];
		this.keyInfoArray = new KeyInfo[this.certificates.length];

		SignatureMethod signatureMethod = this.xmlSignatureFactory
				.newSignatureMethod(this.signatureAlgorithm, null);

		CanonicalizationMethod canonicalizationMethod = this.xmlSignatureFactory
				.newCanonicalizationMethod(CanonicalizationMethod.INCLUSIVE,
						(C14NMethodParameterSpec) null);

		for (int i = 0; i < this.certificates.length; i++) {

			// Create the SignedInfo.
			SignedInfo actualSi = this.xmlSignatureFactory.newSignedInfo(
					canonicalizationMethod, signatureMethod,
					Collections.singletonList(ref));

			// Create the KeyInfo containing the X509Data.
			KeyInfoFactory kif = this.xmlSignatureFactory.getKeyInfoFactory();
			List<Serializable> x509Content = new ArrayList<Serializable>();
			x509Content.add(this.certificates[i].getSubjectX500Principal()
					.getName());
			x509Content.add(this.certificates[i]);
			X509Data xd = kif.newX509Data(x509Content);
			KeyInfo actualKi = kif.newKeyInfo(Collections.singletonList(xd));

			this.signedInfoArray[i] = actualSi;
			this.keyInfoArray[i] = actualKi;
		}
	}

	/**
	 * Method used to sign a content of file. It returns the content of file
	 * signed.
	 * 
	 * @param data
	 *            The text to sign
	 * @throws ParserConfigurationException
	 * @throws IOException
	 * @throws SAXException
	 * @throws XMLSignatureException
	 * @throws MarshalException
	 * @throws TransformerException
	 */

	public void signText(StringBuilder data) throws SAXException, IOException,
			ParserConfigurationException, MarshalException,
			XMLSignatureException, TransformerException {

		Writer finalWriter = new StringWriter();

		try {
			// Instantiate the document to be signed.
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);

			Document doc = dbf.newDocumentBuilder().parse(
					new InputSource(new StringReader(data.toString())));

			data.setLength(0);

			Element rootElement = doc.getDocumentElement();

			for (int i = 0; i < this.certificates.length; i++) {

				// Create a DOMSignContext and specify the RSA PrivateKey and
				// location of the resulting XMLSignature's parent element.
				DOMSignContext dsc = new DOMSignContext(this.privateKeys[i],
						rootElement);

				// Create the XMLSignature, but don't sign it yet.
				XMLSignature signature = this.xmlSignatureFactory
						.newXMLSignature(this.signedInfoArray[i],
								this.keyInfoArray[i]);

				dsc.setBaseURI("file:///");

				// Marshal, generate, and sign the enveloped signature.
				signature.sign(dsc);
			}

			// Output the resulting document.
			StreamResult result = new StreamResult(finalWriter);
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer trans = tf.newTransformer();
			trans.transform(new DOMSource(doc), result);

			data.append(result.getWriter().toString());

		} finally {
			finalWriter.close();
		}
	}
}