/**
 * SignValidator.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.security.sign;

import java.io.IOException;
import java.io.StringReader;
import java.security.PublicKey;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.util.Iterator;

import javax.xml.crypto.dsig.Reference;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.XMLValidateContext;
import javax.xml.crypto.dsig.dom.DOMValidateContext;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.xml.security.Init;
import org.apache.xml.security.exceptions.XMLSecurityException;
import org.apache.xml.security.utils.Constants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Class used to validate signatures.
 */
public class SignValidator {

	static {
		Security.insertProviderAt(
				new org.apache.jcp.xml.dsig.internal.dom.XMLDSigRI(), 1);
	}

	/**
	 * Method used to validate a digital sign in a content of file. Also return
	 * the certificates used to signing the file.
	 * 
	 * @param data
	 *            XML data to validate.
	 * 
	 * @return Amount of validated certificates.
	 * 
	 * @throws SAXException
	 *             the sAX exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws ParserConfigurationException
	 *             the parser configuration exception
	 * @throws TransformerException
	 *             the transformer exception
	 * @throws XMLSignatureException
	 *             the xML signature exception
	 * @throws XMLSecurityException
	 *             the xML security exception
	 */
	public static X509Certificate[] validateSignAndGetX509Certificates(
			String data) throws SAXException, IOException,
			ParserConfigurationException, TransformerException,
			XMLSignatureException, XMLSecurityException {

		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

		dbf.setNamespaceAware(true);

		Document doc = dbf.newDocumentBuilder().parse(
				new InputSource(new StringReader(data)));

		// Find Signature element.
		NodeList nl = doc.getElementsByTagNameNS(Constants.SignatureSpecNS,
				"Signature");

		if (nl.getLength() == 0) {
			throw new RuntimeException("Cannot find Signature element");
		}

		Init.init();

		X509Certificate[] x509Certificates = new X509Certificate[nl.getLength()];

		for (int i = 0; i < nl.getLength(); i++) {
			org.apache.xml.security.signature.XMLSignature signature = new org.apache.xml.security.signature.XMLSignature(
					(Element) nl.item(0), "");

			org.apache.xml.security.keys.KeyInfo keyInfo = signature
					.getKeyInfo();

			if (keyInfo == null) {
				throw new RuntimeException("No keyinfo");
			}

			X509Certificate certificate = keyInfo.getX509Certificate();

			if (certificate == null) {
				throw new RuntimeException("No certificate");
			}

			if (signature.checkSignatureValue(certificate)) {
				x509Certificates[i] = certificate;
			} else {
				return null;
			}
		}

		return x509Certificates;
	}

	/**
	 * Method used to validate a digital sign in a content of file. Also return
	 * the certificate used to signing the file.
	 * 
	 * @param data
	 *            XML text to validate the signature
	 * 
	 * @return XML text signed
	 * 
	 * @throws SAXException
	 *             If a problem parsering XML occurs
	 * @throws IOException
	 *             If a problem parsering XML occurs
	 * @throws ParserConfigurationException
	 *             If a problem parsering XML occurs
	 * @throws TransformerException
	 *             the transformer exception
	 * @throws XMLSignatureException
	 *             the xML signature exception
	 * @throws XMLSecurityException
	 *             If a problem in one or more of the signing parameters occurs
	 */
	public static X509Certificate validateSignAndGetX509Certificate(String data)
			throws SAXException, IOException, ParserConfigurationException,
			TransformerException, XMLSignatureException, XMLSecurityException {

		X509Certificate[] x509Certificates = validateSignAndGetX509Certificates(data);

		if (x509Certificates != null) {
			return x509Certificates[0];
		} else {
			return null;
		}
	}

	/**
	 * Method used to validate a digital sign in a content of file.
	 * 
	 * @param data
	 *            XML data to validate.
	 * 
	 * @return Amount of validated certificates.
	 * 
	 * @throws SAXException
	 *             the sAX exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws ParserConfigurationException
	 *             the parser configuration exception
	 * @throws TransformerException
	 *             the transformer exception
	 * @throws XMLSignatureException
	 *             the xML signature exception
	 * @throws XMLSecurityException
	 *             the xML security exception
	 */
	public static int getCertificatesValidated(String data)
			throws SAXException, IOException, ParserConfigurationException,
			TransformerException, XMLSignatureException, XMLSecurityException {

		X509Certificate[] x509Certificates = validateSignAndGetX509Certificates(data);

		if (x509Certificates != null) {
			return x509Certificates.length;
		} else {
			return 0;
		}
	}

	/**
	 * Method used to validate a digital sign in a content of file.
	 * 
	 * @param data
	 *            XML data to validate
	 * 
	 * @return True if the signature is valid.
	 * 
	 * @throws SAXException
	 *             the sAX exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws ParserConfigurationException
	 *             the parser configuration exception
	 * @throws TransformerException
	 *             the transformer exception
	 * @throws XMLSignatureException
	 *             the xML signature exception
	 * @throws XMLSecurityException
	 *             the xML security exception
	 */
	public static boolean validateSignature(String data) throws SAXException,
			IOException, ParserConfigurationException, TransformerException,
			XMLSignatureException, XMLSecurityException {

		X509Certificate x509Certificate = validateSignAndGetX509Certificate(data);

		if (x509Certificate != null) {
			return true;
		}

		return false;
	}

	/**
	 * Method used to validate detached XML signature.
	 * 
	 * @param data
	 *            Signed data to validate.
	 * @param publicKey
	 *            Certificate public key used to sign.
	 * @return Amount of validated certificates.
	 * 
	 * @throws Exception
	 *             If any problem occurs
	 * 
	 */
	public static int validateSign(String data, PublicKey publicKey)
			throws Exception {
		boolean allValidSigns = true;
		int signsChecked = 0;
		// Instantiate the document to be signed.
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		Document doc = dbf.newDocumentBuilder().parse(
				new InputSource(new StringReader(data)));
		// Find Signature element.
		NodeList nl = doc.getElementsByTagNameNS(XMLSignature.XMLNS,
				"Signature");

		if (nl.getLength() == 0) {
			throw new Exception("Cannot find Signature element");
		}

		XMLSignatureFactory fac = XMLSignatureFactory.getInstance("DOM");

		for (int i = 0; i < nl.getLength(); i++) {
			// Create a DOMValidateContext and specify a KeySelector
			// and document context.
			DOMValidateContext valContext = new DOMValidateContext(
					new X509KeySelector(), nl.item(i));
			// Unmarshal the XMLSignature.
			XMLSignature signature = fac.unmarshalXMLSignature(valContext);
			// Validate the XMLSignature.
			// boolean coreValidity = signature.validate(valContext);
			boolean coreValidity = signature.getSignatureValue().validate(
					valContext);

			XMLValidateContext vc = new DOMValidateContext(publicKey,
					doc.getDocumentElement());

			// Check core validation status
			if (coreValidity == false) {
				// check the validation status of each Reference
				Iterator<?> iterator = signature.getSignedInfo()
						.getReferences().iterator();
				while (iterator.hasNext()) {
					Reference reference = (Reference) iterator.next();
					reference.validate(vc);
				}
			}

			// You can also validate an XML Signature which is in XML format.
			// Unmarshal and validate an XMLSignature from a DOMValidateContext
			signature = fac.unmarshalXMLSignature(vc);

			allValidSigns &= coreValidity;
			signsChecked++;
		}

		if (allValidSigns) {
			return signsChecked;
		} else {
			return 0;
		}
	}
}