/**
 * X509KeySelector.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.security.sign;

import java.security.Key;
import java.security.PublicKey;
import java.security.cert.X509Certificate;
import java.util.Iterator;

import javax.xml.crypto.AlgorithmMethod;
import javax.xml.crypto.KeySelector;
import javax.xml.crypto.KeySelectorException;
import javax.xml.crypto.KeySelectorResult;
import javax.xml.crypto.XMLCryptoContext;
import javax.xml.crypto.XMLStructure;
import javax.xml.crypto.dsig.SignatureMethod;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import javax.xml.crypto.dsig.keyinfo.X509Data;

/**
 * Class used to obtain the public key using the information in the KeyInfo
 * element and giving back it to be used like validation key.
 * 
 * @version 1.0 (08/04/2008)
 * @author Héctor Rodríguez (hector.rodriguez@smartmatic.com)
 */

public class X509KeySelector extends KeySelector {

	/** The certificate. */
	private X509Certificate certificate;

	/**
	 * It obtains the public key from the first X.509 certificate that finds in
	 * the X509Data.
	 * 
	 * Attempts to find a key that satisfies the specified constraints.
	 * 
	 * @param keyInfo
	 *            a KeyInfo (may be null)
	 * @param purpose
	 *            the key's purpose (KeySelector.Purpose.SIGN,
	 *            KeySelector.Purpose.VERIFY, KeySelector.Purpose.ENCRYPT, or
	 *            KeySelector.Purpose.DECRYPT)
	 * @param method
	 *            the algorithm method that this key is to be used for. Only
	 *            keys that are compatible with the algorithm and meet the
	 *            constraints of the specified algorithm should be returned.
	 * @param context
	 *            an XMLCryptoContext that may contain useful information for
	 *            finding an appropriate key. If this key selector supports
	 *            resolving RetrievalMethod types, the context's baseURI and
	 *            dereferencer parameters (if specified) should be used by the
	 *            selector to resolve and dereference the URI.
	 * @return the result of the key selector
	 * @throws KeySelectorException
	 *             if an exceptional condition occurs while attempting to find a
	 *             key. Note that an inability to find a key is not considered
	 *             an exception (null should be returned in that case). However,
	 *             an error condition (ex: network communications failure) that
	 *             prevented the KeySelector from finding a potential key should
	 *             be considered an exception.
	 */
	@Override
	public KeySelectorResult select(KeyInfo keyInfo,
			KeySelector.Purpose purpose, AlgorithmMethod method,
			XMLCryptoContext context) throws KeySelectorException {
		Iterator ki = keyInfo.getContent().iterator();
		while (ki.hasNext()) {
			XMLStructure info = (XMLStructure) ki.next();
			if (!(info instanceof X509Data)) {
				continue;
			}
			X509Data x509Data = (X509Data) info;
			Iterator xi = x509Data.getContent().iterator();
			while (xi.hasNext()) {
				Object o = xi.next();
				if (!(o instanceof X509Certificate)) {
					continue;
				}
				final PublicKey key = ((X509Certificate) o).getPublicKey();
				// Make sure the algorithm is compatible
				// with the method.
				this.certificate = (X509Certificate) o;
				if (algEquals(method.getAlgorithm(), key.getAlgorithm())) {
					return new KeySelectorResult() {
						@Override
						public Key getKey() {
							return key;
						}
					};
				}
			}
		}
		throw new KeySelectorException("No key found!");
	}

	/**
	 * Only test for some algorithms. Not all of them.
	 * 
	 * @param algURI
	 *            Algorithm URI
	 * @param algName
	 *            Algorithm name
	 * @return If the algorithm is equals
	 */
	static boolean algEquals(String algURI, String algName) {
		if ((algName.equalsIgnoreCase("DSA") && algURI
				.equalsIgnoreCase(SignatureMethod.DSA_SHA1))
				|| (algName.equalsIgnoreCase("RSA") && algURI
						.equalsIgnoreCase(SignatureMethod.RSA_SHA1))) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Getter of <code>certificate</code> property.
	 * 
	 * @return <code>certificate</code> property value.
	 * @see #certificate
	 */

	public X509Certificate getCertificate() {
		return this.certificate;
	}
}
