/**
 * PoolableSymmetricCipher.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.security.symmetric;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.AlgorithmParameterSpec;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.IOUtils;

import com.smartmatic.waffle.security.io.Base64Encoder;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * <p>
 * Description: Class used to encrypt and decrypt information using a symmetric
 * algorithm. In the methods that this class receives or returns a encrypted
 * String is supposed that the String is encoded in Base 64.
 * </p>
 * <p>
 * This class interacts with all the cryptographic objects using the Java
 * Cryptographic Extension (JCE). Using the JCE specification the change of the
 * cryptographic algorithm is done changing the name of the algorithms
 * </p>
 * <p>
 * This class always returns and expects printable strings. In the encryption
 * process after the codification of the information, the result byte array is
 * converted into a Base64 String. In the decryption process before START the
 * decryption process this class convert the Base64 String into a byte array
 * that is passed to the cipher to decrypt.
 * </p>
 * 
 * @version 1.0
 * @author Ithamar Torres
 */

public class PoolableSymmetricCipher {

	/**
	 * Field used to indicate to this class which Cipher Block chaining mode
	 * will be used in the encryption. This field is used when the Cipher Block
	 * Chaining is desired.
	 */

	public final static byte CBC = 0;

	/**
	 * Field used to indicate to this class which Cipher Block chaining mode
	 * will be used in the encryption. This field is used when the Electronic
	 * Code Bock is desired as the chaining mode.
	 */

	public final static byte ECB = 1;

	/**
	 * Criptographic Algorithm that will be used in the encryption / decryption
	 * process.
	 */

	// protected final static String KEYALG = "DesEde";
	protected final static String KEYALG = "AES";

	/**
	 * Extended name of the algorithm that will be used in the encryption /
	 * decryption process. The format to define the extended name of the
	 * algorithm is: Algorithm/Mode/Padding. This is the extended algorithm
	 * definition used when the Cipher Block chaining is used as output cipher
	 * mode </p>
	 * 
	 * @see <a href=
	 *      "http://java.sun.com/j2se/1.4.2/docs/guide/security/jce/JCERefGuide.html#CipherClass"
	 *      > Java Cryptographic Extension< /a>
	 */

	/**
	 * Its really important to know that its NOT THE SAME to use PKCS5 and PKCS7
	 * Padding. The theory says it so. But in practice there is a interoperating
	 * problem with .NET and the same padding (PKCS7 Padding). Conclussion if
	 * you want to use AES(Rijndael) with CBC use in java PKCS5Padding and in
	 * .NET PKCS7Padding
	 */
	public final static String ALGORITHM_CBC = PoolableSymmetricCipher.KEYALG
			+ "/CBC/PKCS5Padding";

	/**
	 * Extended name of the algorithm that will be used in the encryption /
	 * decryption process. The format to define the extended name of the
	 * algorithm is: Algorithm/Mode/Padding. TThis is the extended algorithm
	 * definition used when the Electronic Code Book is used </p>
	 * 
	 * @see <a href=
	 *      "http://java.sun.com/j2se/1.4.2/docs/guide/security/jce/JCERefGuide.html#CipherClass"
	 *      > Java Cryptographic Extension< /a>
	 */

	public final static String ALGORITHM_ECB = PoolableSymmetricCipher.KEYALG
			+ "/ECB/PKCS5Padding";

	/**
	 * Object used to encode the information that this object has to encrypt.
	 */

	protected Cipher encodingCipher;

	/** Object used to decrypt the information that needs to be decoded. */

	protected Cipher decodingCipher;

	/**
	 * Secret key used by the encoding and decoding ciphers to encrypt and
	 * decrypt the messages received.
	 */

	/**
	 * The initialization vector for CBC encryption.
	 */
	protected byte[] iv;

	protected SecretKey secretKey;

	/**
	 * This constructor creates a Symmetric Cipher with a output feedback mode
	 * of ECB. Because ECB does not need an inicialization Vector this
	 * constructor is different from the one that creates a CBC symmetric cipher
	 * by the customCode of parameters required
	 * 
	 * @param key
	 *            SecretKey Key that will be used in the encryption / decryption
	 *            process
	 * 
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */

	public PoolableSymmetricCipher(SecretKey key)
			throws CryptographicStaticException, CryptographicRuntimeException {
		try {
			this.secretKey = key;

			this.decodingCipher = Cipher
					.getInstance(PoolableSymmetricCipher.ALGORITHM_ECB);
			this.encodingCipher = Cipher
					.getInstance(PoolableSymmetricCipher.ALGORITHM_ECB);

			this.encodingCipher.init(Cipher.ENCRYPT_MODE, this.secretKey);
			this.decodingCipher.init(Cipher.DECRYPT_MODE, this.secretKey);
		} catch (NoSuchPaddingException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.PADDING_OR_PROVIDER_PROBLEM,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (InvalidKeyException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.SYMMETRIC_KEY_PROBLEM,
					e);
		}
	}

	/**
	 * Empty constructor used by subclasses of this class.
	 */

	protected PoolableSymmetricCipher() {
	}

	/**
	 * Constructor that creates a SymmetricCipher with a Output FeedBadck Mode
	 * of CBC. The Inicialization vector required by the algorithm is received
	 * as parameter.
	 * 
	 * @param key
	 *            SecretKey Key that will be used in the encryption / decryption
	 *            process
	 * @param iVector
	 *            String Inizialization Vector used in the encryption /
	 *            decryption. This parameter must be encoded in a Base 64
	 *            String.
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */

	public PoolableSymmetricCipher(SecretKey key, String iVector)
			throws CryptographicStaticException, CryptographicRuntimeException {
		try {
			this.secretKey = key;

			this.decodingCipher = Cipher
					.getInstance(PoolableSymmetricCipher.ALGORITHM_CBC);
			this.encodingCipher = Cipher
					.getInstance(PoolableSymmetricCipher.ALGORITHM_CBC);

			Base64 encoder = new Base64();
			byte[] iVectorData = encoder.decode(iVector.getBytes());

			IvParameterSpec parametersSpec = new IvParameterSpec(iVectorData);

			this.iv = parametersSpec.getIV();

			this.encodingCipher.init(Cipher.ENCRYPT_MODE, this.secretKey,
					parametersSpec);
			this.decodingCipher.init(Cipher.DECRYPT_MODE, this.secretKey,
					parametersSpec);

		} catch (NoSuchPaddingException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.PADDING_OR_PROVIDER_PROBLEM,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (InvalidKeyException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.SYMMETRIC_KEY_PROBLEM,
					e);
		} catch (InvalidAlgorithmParameterException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.SYMMETRIC_KEY_PROBLEM,
					e);
		}
	}

	/**
	 * Instantiates a new poolable symmetric cipher.
	 * 
	 * @param key
	 *            the key
	 * @param iVector
	 *            the i vector
	 * @throws CryptographicStaticException
	 *             the cryptographic static exception
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */
	public PoolableSymmetricCipher(SecretKey key, byte[] iVector)
			throws CryptographicStaticException, CryptographicRuntimeException {
		try {
			this.secretKey = key;

			this.decodingCipher = Cipher
					.getInstance(PoolableSymmetricCipher.ALGORITHM_CBC);
			this.encodingCipher = Cipher
					.getInstance(PoolableSymmetricCipher.ALGORITHM_CBC);

			IvParameterSpec parametersSpec = new IvParameterSpec(iVector);

			this.iv = parametersSpec.getIV();

			this.encodingCipher.init(Cipher.ENCRYPT_MODE, this.secretKey,
					parametersSpec);
			this.decodingCipher.init(Cipher.DECRYPT_MODE, this.secretKey,
					parametersSpec);

		} catch (NoSuchPaddingException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.PADDING_OR_PROVIDER_PROBLEM,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (InvalidKeyException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.SYMMETRIC_KEY_PROBLEM,
					e);
		} catch (InvalidAlgorithmParameterException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.SYMMETRIC_KEY_PROBLEM,
					e);
		}
	}

	/**
	 * Constructor that creates a SymmetricCipher with a Output FeedBadck Mode
	 * of CBC. The Inicialization vector required by the algorithm is received
	 * as parameter.
	 * 
	 * @param key
	 *            SecretKey Key that will be used in the encryption / decryption
	 *            process
	 * @param iVector
	 *            String Inizialization Vector used in the encryption /
	 *            decryption. This parameter must be encoded in a Base 64
	 *            String.
	 * @param algorithm
	 *            String Algorithm used to create a Symmetric Cipher.
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of innappropiate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */

	public PoolableSymmetricCipher(SecretKey key, String iVector,
			String algorithm) throws CryptographicStaticException,
			CryptographicRuntimeException {
		try {
			this.secretKey = key;

			this.decodingCipher = Cipher.getInstance(algorithm);
			this.encodingCipher = Cipher.getInstance(algorithm);

			Base64 encoder = new Base64();
			byte[] iVectorData = encoder.decode(iVector.getBytes());

			AlgorithmParameterSpec parametersSpec = new IvParameterSpec(
					iVectorData);

			this.encodingCipher.init(Cipher.ENCRYPT_MODE, this.secretKey,
					parametersSpec);
			this.decodingCipher.init(Cipher.DECRYPT_MODE, this.secretKey,
					parametersSpec);

		} catch (NoSuchPaddingException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.PADDING_OR_PROVIDER_PROBLEM,
					e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptographicStaticException(
					CryptographicStaticException.ExceptionInfo.ALGORITHM_OR_PROVIDER_PROBLEM,
					e);
		} catch (InvalidKeyException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.SYMMETRIC_KEY_PROBLEM,
					e);
		} catch (InvalidAlgorithmParameterException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.SYMMETRIC_KEY_PROBLEM,
					e);
		}
	}

	/**
	 * Method that encrypts a given message and append the IV at the beginning
	 * of byte array. This method does not convert the result into a printable
	 * string. It returns the encrypted message as a byte array of possible
	 * non-printable characters
	 * 
	 * @param message
	 *            byte[] The message to encrypt
	 * @return byte[] The encrypted message without the transformation into a
	 *         Base64 String
	 * @throws CryptographicRuntimeException
	 *             if occurred a problem reading / writing a resource
	 */
	public byte[] encryptAndAppendIvToBytes(byte[] message)
			throws CryptographicRuntimeException {
		try {
			byte[] encryptedData = this.encodingCipher.doFinal(message);
			byte[] encryptedDataWithIV = new byte[encryptedData.length
					+ this.iv.length];
			System.arraycopy(this.iv, 0, encryptedDataWithIV, 0, this.iv.length);
			System.arraycopy(encryptedData, 0, encryptedDataWithIV,
					this.iv.length, encryptedData.length);
			return encryptedDataWithIV;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		}
	}

	/**
	 * Method that encrypts a given message. This method after encrypt the
	 * message transform the result into a printable string converting the
	 * result into a Base64 String.
	 * 
	 * @param message
	 *            String The message to encrypt
	 * @return String the encrypted message converted in a printable Base64
	 *         String.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */

	public String encrypt(String message) throws CryptographicRuntimeException {
		try {

			// Check if is a valid message, if not return null
			if (message == null) {
				return message;
			}

			Base64 base64Encoder = new Base64();

			byte[] data = message.getBytes();
			String cipherText = new String(
					base64Encoder.encode(this.encodingCipher.doFinal(data)));
			return cipherText;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		}
	}

	/**
	 * Method that returns an output stream that encrypts data on the fly.
	 * 
	 * @param messageDestination
	 *            OutputStream where the encrypted data will be written
	 * 
	 * @return CipherOutputStream A CipherOutputStream is composed of an
	 *         OutputStream and a Cipher so that write() methods first process
	 *         the data before writing them out to the underlying OutputStream.
	 *         The cipher must be fully initialized before being used by a
	 *         CipherOutputStream.
	 */

	public CipherOutputStream encryptBytes(OutputStream messageDestination) {

		return new CipherOutputStream(messageDestination, this.encodingCipher);
	}

	/**
	 * Method that decrypts the information contained in a message. This method
	 * assumes that the message after been encrypted was encoded into a Base 64
	 * String. This method before starting the decryption per se decode the
	 * message from a Base 64 String into the encrypted message and later
	 * decrypts the message
	 * 
	 * @param message
	 *            String The message to be decrypted
	 * @return String The decrytped message. After the decryption this method
	 *         converts the byte array that results into a String
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */

	public String decrypt(String message) throws CryptographicRuntimeException {
		Base64 base64Encoder = new Base64();
		byte[] dataDecoded = base64Encoder.decode(message.getBytes());
		return this.decrypt(dataDecoded);
	}

	/**
	 * Method that decrypts the information contained in a message. This method
	 * assumes that the message after been encrypted was encoded into a Base 64
	 * String. This method before starting the decryption per se decode the
	 * message from a Base 64 String into the encrypted message and later
	 * decrypts the message. This method does not convert the byte array that
	 * results from the decryption into a String, instead returns the byte
	 * array.
	 * 
	 * @param message
	 *            String The message to be decrypted
	 * @return byte[] The result of the decryption without the conversion of the
	 *         byte array into a String.
	 * @throws CryptographicRuntimeException
	 *             if ocurred a problem reading / writing a resource
	 */

	public byte[] decryptBytes(String message)
			throws CryptographicRuntimeException {
		try {
			Base64 base64Encoder = new Base64();
			byte[] dataDecoded = base64Encoder.decode(message.getBytes());
			byte[] dataPlain = this.decodingCipher.doFinal(dataDecoded);
			return dataPlain;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		}
	}

	/**
	 * Method that decrypts the information contained in a message. This method
	 * assumes that the message received does not have been converted into a
	 * Base 64 String
	 * 
	 * @param encData
	 *            byte[] The message to be encrypted
	 * @return String The decrytped message. After the decryption this method
	 * @throws CryptographicRuntimeException
	 *             If there is a problem in the process of encrypt / decrypt the
	 *             information
	 */

	public byte[] decryptBytes(byte[] encData)
			throws CryptographicRuntimeException {
		try {
			byte[] dataPlain = this.decodingCipher.doFinal(encData);
			return dataPlain;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		}
	}

	/**
	 * Method that decrypts the information contained in a message. This method
	 * assumes that the message received does not have been converted into a
	 * Base 64 String
	 * 
	 * @param message
	 *            byte[] The message to be decrypted
	 * @return String The decrytped message. After the decryption this method
	 *         converts the byte array that results into a String
	 * @throws CryptographicRuntimeException
	 *             If there is a problem in the process of encrypt / decrypt the
	 *             information
	 */

	public String decrypt(byte[] message) throws CryptographicRuntimeException {
		try {
			byte[] dataPlain = this.decodingCipher.doFinal(message);
			String plainText = new String(dataPlain);
			return plainText;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		}
	}

	/**
	 * Method that decrypts the information contained in a message. This method
	 * assumes that the message received does not have been converted into a
	 * Base 64 String
	 * 
	 * @param message
	 *            InputStream The message to be decrypted
	 * 
	 * @return CipherInputStream A CipherInputStream is composed of an
	 *         InputStream and a Cipher so that read() methods return data that
	 *         are read in from the underlying InputStream but have been
	 *         additionally processed by the Cipher.
	 */

	public CipherInputStream decrypt(InputStream message) {
		return new CipherInputStream(message, this.decodingCipher);
	}

	/**
	 * Method that encrypts a given message. This method does not convert the
	 * result into a printable string. It returns the encrypted message as a
	 * byte array of possible non-printable characters
	 * 
	 * @param message
	 *            byte[] The message to encrypt
	 * @return byte[] The encrypted message without the transformation into a
	 *         Base64 String
	 * @throws CryptographicRuntimeException
	 *             if occurred a problem reading / writing a resource
	 */

	public byte[] encryptBytes(byte[] message)
			throws CryptographicRuntimeException {
		try {
			return this.encodingCipher.doFinal(message);
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		}
	}

	/**
	 * Method that encrypts a object with an asymmetric algorithm using the key
	 * received in the constructor and encrypt data using base 64 algorithm.
	 * 
	 * @param message
	 *            message's bytes representation
	 * @return String message in base 64 representation
	 * @throws CryptographicRuntimeException
	 *             if occurred a problem reading / writing a resource
	 */

	public String encryptBytesBase64(byte[] message)
			throws CryptographicRuntimeException {

		byte[] data = this.encryptBytes(message);
		Base64 base64Encoder = new Base64();

		String cipherText = new String(base64Encoder.encode(data));
		return cipherText;
	}

	/**
	 * Method that decrypt an input stream returning its bytes.
	 * 
	 * @param input
	 *            Input stream to be ciphered
	 * @return Bytes ciphered
	 * @throws IOException
	 *             An error reading the stream
	 */

	public byte[] decryptToBytes(InputStream input) throws IOException {
		InputStream decryptedInput = new CipherInputStream(input,
				this.decodingCipher);

		return IOUtils.toByteArray(decryptedInput);
	}

	/**
	 * <p>
	 * Method that encrypts a object with an asymmetric algorithm using the key
	 * received in the constructor
	 * </p>
	 * .
	 * 
	 * @param message
	 *            Serializable The Object that needs to be encrypted
	 * @return byte[] The encrypted object. The result of the encryption is a
	 *         byte array that represents the serialized and encrypted object.
	 * @throws CryptographicRuntimeException
	 *             the cryptographic runtime exception
	 */
	public byte[] encryptObject(Serializable message)
			throws CryptographicRuntimeException {

		ByteArrayOutputStream aos = null;
		ObjectOutputStream oos = null;

		try {
			// Check if is a valid message, if not return null
			if (message == null) {
				return null;
			}

			// Create a Byte Array that will be cipher
			aos = new ByteArrayOutputStream();
			oos = new ObjectOutputStream(aos);
			oos.writeObject(message);
			byte[] data = aos.toByteArray();

			// Cipher the byte array
			byte[] cipherObject = this.encodingCipher.doFinal(data);

			return cipherObject;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		} finally {
			try {
				oos.close();
			} catch (IOException e) {
			}
		}
	}

	/**
	 * <p>
	 * Method that decrypts an object encrypted with an asymmetric algorithm
	 * using the key received in the constructor
	 * </p>
	 * 
	 * @param object
	 *            String The object that needs to be decrypted
	 * @return Serializable The plain message.
	 * @throws CryptographicRuntimeException
	 *             If there is a problem in the process of encrypt / decrypt the
	 *             information
	 */

	public Serializable decryptObject(byte[] object)
			throws CryptographicRuntimeException {

		ByteArrayInputStream bais = null;
		ObjectInputStream ois = null;
		byte[] dataPlain = null;

		try {
			dataPlain = this.decodingCipher.doFinal(object);

			bais = new ByteArrayInputStream(dataPlain);
			ois = new ObjectInputStream(bais);

			return (Serializable) ois.readObject();

		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IOException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.IO_PROBLEM, e);
		} catch (ClassNotFoundException e) {
			// TODO agregar mensajes de error ClassNotFoundException
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.UNCATEGORIZED_PROBLEM,
					e);
		} finally {
			if (ois != null) {
				try {
					ois.close();
				} catch (IOException e) {
					// Do nothing...
				}
			}
		}

	}

	/**
	 * Method that encrypts a given message. This method after encrypt the
	 * message transform the result into a printable string converting the
	 * result into a Base64 String.
	 * 
	 * @param message
	 *            byte[] The message to encrypt
	 * @return String the encrypted message converted in a printable Base64
	 *         String.
	 * @throws CryptographicRuntimeException
	 *             If there is a problem in the proccess of encrypt / decrypt
	 *             the information
	 */

	public String encrypt(byte[] message) throws CryptographicRuntimeException {
		try {
			String cipherText = new String(
					Base64Encoder.encode(this.encodingCipher.doFinal(message)));
			return cipherText;
		} catch (BadPaddingException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptographicRuntimeException(
					CryptographicRuntimeException.ExceptionInfo.INVALID_MESSAGE_FORMAT,
					e);
		}
	}
}